@@ -105,7 +105,9 @@ func TestResourceProvider_Validate_bad_to_many_src(t *testing.T) {
 // Validate that Stop can Close can be called even when not provisioning.
 func TestResourceProvisioner_StopClose(t *testing.T) {
 	p := New()
-	p.Stop()
+	if err := p.Stop(); err != nil {
+		t.Fatal(err)
+	}
 	p.Close()
 }
 
