@@ -36,16 +36,16 @@ import (
 	"github.com/opentofu/opentofu/internal/tofu"
 )
 
-func testOperationPlan(t *testing.T, configDir string) (*backend.Operation, func(), func(*testing.T) *terminal.TestOutput) {
+func testOperationPlan(t *testing.T, configDir string) (*backend.Operation, func(*testing.T) *terminal.TestOutput) {
 	t.Helper()
 
 	return testOperationPlanWithTimeout(t, configDir, 0)
 }
 
-func testOperationPlanWithTimeout(t *testing.T, configDir string, timeout time.Duration) (*backend.Operation, func(), func(*testing.T) *terminal.TestOutput) {
+func testOperationPlanWithTimeout(t *testing.T, configDir string, timeout time.Duration) (*backend.Operation, func(*testing.T) *terminal.TestOutput) {
 	t.Helper()
 
-	_, configLoader, configCleanup := initwd.MustLoadConfigForTests(t, configDir, "tests")
+	_, configLoader := initwd.MustLoadConfigForTests(t, configDir, "tests")
 
 	streams, done := terminal.StreamsForTesting(t)
 	view := views.NewView(streams)
@@ -65,15 +65,14 @@ func testOperationPlanWithTimeout(t *testing.T, configDir string, timeout time.D
 		Type:            backend.OperationTypePlan,
 		View:            operationView,
 		DependencyLocks: depLocks,
-	}, configCleanup, done
+	}, done
 }
 
 func TestCloud_planBasic(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan")
 	defer done(t)
 
 	op.Workspace = testBackendSingleWorkspaceName
@@ -99,7 +98,7 @@ func TestCloud_planBasic(t *testing.T) {
 		t.Fatalf("expected plan summary in output: %s", output)
 	}
 
-	stateMgr, _ := b.StateMgr(testBackendSingleWorkspaceName)
+	stateMgr, _ := b.StateMgr(t.Context(), testBackendSingleWorkspaceName)
 	// An error suggests that the state was not unlocked after the operation finished
 	if _, err := stateMgr.Lock(statemgr.NewLockInfo()); err != nil {
 		t.Fatalf("unexpected error locking state after successful plan: %s", err.Error())
@@ -117,8 +116,7 @@ func TestCloud_planJSONBasic(t *testing.T) {
 		Colorize: mockColorize(),
 	}
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan-json-basic")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan-json-basic")
 	defer done(t)
 
 	op.Workspace = testBackendSingleWorkspaceName
@@ -145,7 +143,7 @@ func TestCloud_planJSONBasic(t *testing.T) {
 		t.Fatalf("expected plan summary in output: %s", gotOut)
 	}
 
-	stateMgr, _ := b.StateMgr(testBackendSingleWorkspaceName)
+	stateMgr, _ := b.StateMgr(t.Context(), testBackendSingleWorkspaceName)
 	// An error suggests that the state was not unlocked after the operation finished
 	if _, err := stateMgr.Lock(statemgr.NewLockInfo()); err != nil {
 		t.Fatalf("unexpected error locking state after successful plan: %s", err.Error())
@@ -156,8 +154,7 @@ func TestCloud_planCanceled(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan")
 	defer done(t)
 
 	op.Workspace = testBackendSingleWorkspaceName
@@ -175,7 +172,7 @@ func TestCloud_planCanceled(t *testing.T) {
 		t.Fatal("expected plan operation to fail")
 	}
 
-	stateMgr, _ := b.StateMgr(testBackendSingleWorkspaceName)
+	stateMgr, _ := b.StateMgr(t.Context(), testBackendSingleWorkspaceName)
 	// An error suggests that the state was not unlocked after the operation finished
 	if _, err := stateMgr.Lock(statemgr.NewLockInfo()); err != nil {
 		t.Fatalf("unexpected error locking state after cancelled plan: %s", err.Error())
@@ -186,8 +183,7 @@ func TestCloud_planLongLine(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan-long-line")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan-long-line")
 	defer done(t)
 
 	op.Workspace = testBackendSingleWorkspaceName
@@ -225,8 +221,7 @@ func TestCloud_planJSONFull(t *testing.T) {
 		Colorize: mockColorize(),
 	}
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan-json-full")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan-json-full")
 	defer done(t)
 
 	op.Workspace = testBackendSingleWorkspaceName
@@ -257,7 +252,7 @@ func TestCloud_planJSONFull(t *testing.T) {
 		t.Fatalf("expected plan summary in output: %s", gotOut)
 	}
 
-	stateMgr, _ := b.StateMgr(testBackendSingleWorkspaceName)
+	stateMgr, _ := b.StateMgr(t.Context(), testBackendSingleWorkspaceName)
 	// An error suggests that the state was not unlocked after the operation finished
 	if _, err := stateMgr.Lock(statemgr.NewLockInfo()); err != nil {
 		t.Fatalf("unexpected error locking state after successful plan: %s", err.Error())
@@ -281,8 +276,7 @@ func TestCloud_planWithoutPermissions(t *testing.T) {
 	}
 	w.Permissions.CanQueueRun = false
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan")
 
 	op.Workspace = "prod"
 
@@ -307,8 +301,7 @@ func TestCloud_planWithParallelism(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan")
 
 	if b.ContextOpts == nil {
 		b.ContextOpts = &tofu.ContextOpts{}
@@ -337,8 +330,7 @@ func TestCloud_planWithPlan(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan")
 
 	op.PlanFile = planfile.NewWrappedLocal(&planfile.Reader{})
 	op.Workspace = testBackendSingleWorkspaceName
@@ -367,8 +359,7 @@ func TestCloud_planWithPath(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan")
 	defer done(t)
 
 	tmpDir := t.TempDir()
@@ -426,8 +417,7 @@ func TestCloud_planWithoutRefresh(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan")
 	defer done(t)
 
 	op.PlanRefresh = false
@@ -463,8 +453,7 @@ func TestCloud_planWithRefreshOnly(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan")
 	defer done(t)
 
 	op.PlanMode = plans.RefreshOnlyMode
@@ -523,8 +512,7 @@ func TestCloud_planWithTarget(t *testing.T) {
 		}
 	}
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan")
 	defer done(t)
 
 	addr, _ := addrs.ParseAbsResourceStr("null_resource.foo")
@@ -570,8 +558,7 @@ func TestCloud_planWithExclude(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan")
 
 	addr, _ := addrs.ParseAbsResourceStr("null_resource.foo")
 
@@ -602,8 +589,7 @@ func TestCloud_planWithReplace(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan")
 	defer done(t)
 
 	addr, _ := addrs.ParseAbsResourceInstanceStr("null_resource.foo")
@@ -641,8 +627,7 @@ func TestCloud_planWithRequiredVariables(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan-variables")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan-variables")
 	defer done(t)
 
 	op.Variables = testVariables(tofu.ValueFromCLIArg, "foo") // "bar" variable defined in config is  missing
@@ -670,8 +655,7 @@ func TestCloud_planNoConfig(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/empty")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/empty")
 
 	op.Workspace = testBackendSingleWorkspaceName
 
@@ -699,8 +683,7 @@ func TestCloud_planNoChanges(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan-no-changes")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan-no-changes")
 	defer done(t)
 
 	op.Workspace = testBackendSingleWorkspaceName
@@ -735,8 +718,7 @@ func TestCloud_planForceLocal(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan")
 	defer done(t)
 
 	op.Workspace = testBackendSingleWorkspaceName
@@ -771,8 +753,7 @@ func TestCloud_planWithoutOperationsEntitlement(t *testing.T) {
 	b, bCleanup := testBackendNoOperations(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan")
 	defer done(t)
 
 	op.Workspace = testBackendSingleWorkspaceName
@@ -821,8 +802,7 @@ func TestCloud_planWorkspaceWithoutOperations(t *testing.T) {
 		t.Fatalf("error creating named workspace: %v", err)
 	}
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan")
 	defer done(t)
 
 	op.Workspace = "no-operations"
@@ -880,8 +860,7 @@ func TestCloud_planLockTimeout(t *testing.T) {
 		t.Fatalf("error creating pending run: %v", err)
 	}
 
-	op, configCleanup, done := testOperationPlanWithTimeout(t, "./testdata/plan", 50)
-	defer configCleanup()
+	op, done := testOperationPlanWithTimeout(t, "./testdata/plan", 50)
 	defer done(t)
 
 	input := testInput(t, map[string]string{
@@ -928,8 +907,7 @@ func TestCloud_planDestroy(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan")
 	defer done(t)
 
 	op.PlanMode = plans.DestroyMode
@@ -953,8 +931,7 @@ func TestCloud_planDestroyNoConfig(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/empty")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/empty")
 	defer done(t)
 
 	op.PlanMode = plans.DestroyMode
@@ -988,8 +965,7 @@ func TestCloud_planWithWorkingDirectory(t *testing.T) {
 		t.Fatalf("error configuring working directory: %v", err)
 	}
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan-with-working-directory/tofu")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan-with-working-directory/tofu")
 	defer done(t)
 
 	op.Workspace = testBackendSingleWorkspaceName
@@ -1033,22 +1009,13 @@ func TestCloud_planWithWorkingDirectoryFromCurrentPath(t *testing.T) {
 		t.Fatalf("error configuring working directory: %v", err)
 	}
 
-	wd, err := os.Getwd()
-	if err != nil {
-		t.Fatalf("error getting current working directory: %v", err)
-	}
-
 	// We need to change into the configuration directory to make sure
 	// the logic to upload the correct slug is working as expected.
-	if err := os.Chdir("./testdata/plan-with-working-directory/tofu"); err != nil {
-		t.Fatalf("error changing directory: %v", err)
-	}
-	defer os.Chdir(wd) // Make sure we change back again when were done.
+	t.Chdir("./testdata/plan-with-working-directory/tofu")
 
 	// For this test we need to give our current directory instead of the
 	// full path to the configuration as we already changed directories.
-	op, configCleanup, done := testOperationPlan(t, ".")
-	defer configCleanup()
+	op, done := testOperationPlan(t, ".")
 	defer done(t)
 
 	op.Workspace = testBackendSingleWorkspaceName
@@ -1079,8 +1046,7 @@ func TestCloud_planCostEstimation(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan-cost-estimation")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan-cost-estimation")
 	defer done(t)
 
 	op.Workspace = testBackendSingleWorkspaceName
@@ -1114,8 +1080,7 @@ func TestCloud_planPolicyPass(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan-policy-passed")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan-policy-passed")
 	defer done(t)
 
 	op.Workspace = testBackendSingleWorkspaceName
@@ -1149,8 +1114,7 @@ func TestCloud_planPolicyHardFail(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan-policy-hard-failed")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan-policy-hard-failed")
 
 	op.Workspace = testBackendSingleWorkspaceName
 
@@ -1189,8 +1153,7 @@ func TestCloud_planPolicySoftFail(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan-policy-soft-failed")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan-policy-soft-failed")
 
 	op.Workspace = testBackendSingleWorkspaceName
 
@@ -1229,8 +1192,7 @@ func TestCloud_planWithRemoteError(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan-with-error")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan-with-error")
 	defer done(t)
 
 	op.Workspace = testBackendSingleWorkspaceName
@@ -1269,8 +1231,7 @@ func TestCloud_planJSONWithRemoteError(t *testing.T) {
 		Colorize: mockColorize(),
 	}
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan-json-error")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan-json-error")
 	defer done(t)
 
 	op.Workspace = testBackendSingleWorkspaceName
@@ -1302,8 +1263,7 @@ func TestCloud_planOtherError(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan")
 	defer done(t)
 
 	op.Workspace = "network-error" // custom error response in backend_mock.go
@@ -1330,8 +1290,7 @@ func TestCloud_planImportConfigGeneration(t *testing.T) {
 		Colorize: mockColorize(),
 	}
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan-import-config-gen")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan-import-config-gen")
 	defer done(t)
 
 	genPath := filepath.Join(op.ConfigDir, "generated.tf")
@@ -1361,7 +1320,7 @@ func TestCloud_planImportConfigGeneration(t *testing.T) {
 		t.Fatalf("expected plan summary in output: %s", gotOut)
 	}
 
-	stateMgr, _ := b.StateMgr(testBackendSingleWorkspaceName)
+	stateMgr, _ := b.StateMgr(t.Context(), testBackendSingleWorkspaceName)
 	// An error suggests that the state was not unlocked after the operation finished
 	if _, err := stateMgr.Lock(statemgr.NewLockInfo()); err != nil {
 		t.Fatalf("unexpected error locking state after successful plan: %s", err.Error())
@@ -1381,8 +1340,7 @@ func TestCloud_planImportGenerateInvalidConfig(t *testing.T) {
 		Colorize: mockColorize(),
 	}
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan-import-config-gen-validation-error")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan-import-config-gen-validation-error")
 	defer done(t)
 
 	genPath := filepath.Join(op.ConfigDir, "generated.tf")
@@ -1420,8 +1378,7 @@ func TestCloud_planInvalidGenConfigOutPath(t *testing.T) {
 	b, bCleanup := testBackendWithName(t)
 	defer bCleanup()
 
-	op, configCleanup, done := testOperationPlan(t, "./testdata/plan-import-config-gen-exists")
-	defer configCleanup()
+	op, done := testOperationPlan(t, "./testdata/plan-import-config-gen-exists")
 
 	genPath := filepath.Join(op.ConfigDir, "generated.tf")
 	op.GenerateConfigOut = genPath
