@@ -71,12 +71,20 @@ func (s *LocalState) WriteState(state *tofu.State) error {
 	s.mu.Lock()
 	defer s.mu.Unlock()
 
+	err := s.writeState(state)
+	if err != nil {
+		return err
+	}
+
+	// Sync after write
+	return s.stateFileOut.Sync()
+}
+func (s *LocalState) writeState(state *tofu.State) error {
 	if s.stateFileOut == nil {
 		if err := s.createStateFiles(); err != nil {
-			return nil
+			return err
 		}
 	}
-	defer s.stateFileOut.Sync()
 
 	s.state = state.DeepCopy() // don't want mutations before we actually get this written to disk
 
@@ -160,7 +168,9 @@ func (s *LocalState) RefreshState() error {
 		}
 
 		// we have a state file, make sure we're at the start
-		s.stateFileOut.Seek(0, io.SeekStart)
+		if _, err := s.stateFileOut.Seek(0, io.SeekStart); err != nil {
+			return err
+		}
 		reader = s.stateFileOut
 	}
 
