@@ -6,7 +6,6 @@
 package command
 
 import (
-	"os"
 	"strings"
 	"testing"
 
@@ -21,7 +20,7 @@ import (
 func TestGraph(t *testing.T) {
 	td := t.TempDir()
 	testCopyDir(t, testFixturePath("graph"), td)
-	defer testChdir(t, td)()
+	t.Chdir(td)
 
 	ui := new(cli.MockUi)
 	c := &GraphCommand{
@@ -63,7 +62,7 @@ func TestGraph_multipleArgs(t *testing.T) {
 func TestGraph_noArgs(t *testing.T) {
 	td := t.TempDir()
 	testCopyDir(t, testFixturePath("graph"), td)
-	defer testChdir(t, td)()
+	t.Chdir(td)
 
 	ui := new(cli.MockUi)
 	c := &GraphCommand{
@@ -86,8 +85,7 @@ func TestGraph_noArgs(t *testing.T) {
 
 func TestGraph_noConfig(t *testing.T) {
 	td := t.TempDir()
-	os.MkdirAll(td, 0755)
-	defer testChdir(t, td)()
+	t.Chdir(td)
 
 	ui := new(cli.MockUi)
 	c := &GraphCommand{
@@ -106,7 +104,7 @@ func TestGraph_noConfig(t *testing.T) {
 }
 
 func TestGraph_plan(t *testing.T) {
-	testCwd(t)
+	testCwdTemp(t)
 
 	plan := &plans.Plan{
 		Changes: plans.NewChanges(),
