@@ -23,7 +23,7 @@ import (
 )
 
 func TestImport(t *testing.T) {
-	defer testChdir(t, testFixturePath("import-provider-implicit"))()
+	t.Chdir(testFixturePath("import-provider-implicit"))
 
 	statePath := testTempFile(t)
 
@@ -78,7 +78,7 @@ func TestImport(t *testing.T) {
 }
 
 func TestImport_providerConfig(t *testing.T) {
-	defer testChdir(t, testFixturePath("import-provider"))()
+	t.Chdir(testFixturePath("import-provider"))
 
 	statePath := testTempFile(t)
 
@@ -167,7 +167,7 @@ func TestImport_providerConfig(t *testing.T) {
 func TestImport_remoteState(t *testing.T) {
 	td := t.TempDir()
 	testCopyDir(t, testFixturePath("import-provider-remote-state"), td)
-	defer testChdir(t, td)()
+	t.Chdir(td)
 
 	statePath := "imported.tfstate"
 
@@ -279,7 +279,7 @@ func TestImport_remoteState(t *testing.T) {
 func TestImport_initializationErrorShouldUnlock(t *testing.T) {
 	td := t.TempDir()
 	testCopyDir(t, testFixturePath("import-provider-remote-state"), td)
-	defer testChdir(t, td)()
+	t.Chdir(td)
 
 	statePath := "imported.tfstate"
 
@@ -309,7 +309,9 @@ func TestImport_initializationErrorShouldUnlock(t *testing.T) {
 	}
 
 	// overwrite the config with one including a resource from an invalid provider
-	copy.CopyFile(filepath.Join(testFixturePath("import-provider-invalid"), "main.tf"), filepath.Join(td, "main.tf"))
+	if err := copy.CopyFile(filepath.Join(testFixturePath("import-provider-invalid"), "main.tf"), filepath.Join(td, "main.tf")); err != nil {
+		t.Fatal(err)
+	}
 
 	p := testProvider()
 	ui = new(cli.MockUi)
@@ -346,7 +348,7 @@ func TestImport_initializationErrorShouldUnlock(t *testing.T) {
 }
 
 func TestImport_providerConfigWithVar(t *testing.T) {
-	defer testChdir(t, testFixturePath("import-provider-var"))()
+	t.Chdir(testFixturePath("import-provider-var"))
 
 	statePath := testTempFile(t)
 
@@ -426,7 +428,7 @@ func TestImport_providerConfigWithVar(t *testing.T) {
 }
 
 func TestImport_providerConfigWithDataSource(t *testing.T) {
-	defer testChdir(t, testFixturePath("import-provider-datasource"))()
+	t.Chdir(testFixturePath("import-provider-datasource"))
 
 	statePath := testTempFile(t)
 
@@ -491,7 +493,7 @@ func TestImport_providerConfigWithDataSource(t *testing.T) {
 }
 
 func TestImport_providerConfigWithVarDefault(t *testing.T) {
-	defer testChdir(t, testFixturePath("import-provider-var-default"))()
+	t.Chdir(testFixturePath("import-provider-var-default"))
 
 	statePath := testTempFile(t)
 
@@ -570,7 +572,7 @@ func TestImport_providerConfigWithVarDefault(t *testing.T) {
 }
 
 func TestImport_providerConfigWithVarFile(t *testing.T) {
-	defer testChdir(t, testFixturePath("import-provider-var-file"))()
+	t.Chdir(testFixturePath("import-provider-var-file"))
 
 	statePath := testTempFile(t)
 
@@ -650,7 +652,7 @@ func TestImport_providerConfigWithVarFile(t *testing.T) {
 }
 
 func TestImport_emptyConfig(t *testing.T) {
-	defer testChdir(t, testFixturePath("empty"))()
+	t.Chdir(testFixturePath("empty"))
 
 	statePath := testTempFile(t)
 
@@ -682,7 +684,7 @@ func TestImport_emptyConfig(t *testing.T) {
 }
 
 func TestImport_missingResourceConfig(t *testing.T) {
-	defer testChdir(t, testFixturePath("import-missing-resource-config"))()
+	t.Chdir(testFixturePath("import-missing-resource-config"))
 
 	statePath := testTempFile(t)
 
@@ -714,7 +716,7 @@ func TestImport_missingResourceConfig(t *testing.T) {
 }
 
 func TestImport_missingModuleConfig(t *testing.T) {
-	defer testChdir(t, testFixturePath("import-missing-resource-config"))()
+	t.Chdir(testFixturePath("import-missing-resource-config"))
 
 	statePath := testTempFile(t)
 
@@ -748,7 +750,7 @@ func TestImport_missingModuleConfig(t *testing.T) {
 func TestImportModuleVarFile(t *testing.T) {
 	td := t.TempDir()
 	testCopyDir(t, testFixturePath("import-module-var-file"), td)
-	defer testChdir(t, td)()
+	t.Chdir(td)
 
 	statePath := testTempFile(t)
 
@@ -822,7 +824,7 @@ func TestImportModuleVarFile(t *testing.T) {
 func TestImportModuleInputVariableEvaluation(t *testing.T) {
 	td := t.TempDir()
 	testCopyDir(t, testFixturePath("import-module-input-variable"), td)
-	defer testChdir(t, td)()
+	t.Chdir(td)
 
 	statePath := testTempFile(t)
 
@@ -882,7 +884,7 @@ func TestImportModuleInputVariableEvaluation(t *testing.T) {
 }
 
 func TestImport_dataResource(t *testing.T) {
-	defer testChdir(t, testFixturePath("import-missing-resource-config"))()
+	t.Chdir(testFixturePath("import-missing-resource-config"))
 
 	statePath := testTempFile(t)
 
@@ -914,7 +916,7 @@ func TestImport_dataResource(t *testing.T) {
 }
 
 func TestImport_invalidResourceAddr(t *testing.T) {
-	defer testChdir(t, testFixturePath("import-missing-resource-config"))()
+	t.Chdir(testFixturePath("import-missing-resource-config"))
 
 	statePath := testTempFile(t)
 
@@ -946,7 +948,7 @@ func TestImport_invalidResourceAddr(t *testing.T) {
 }
 
 func TestImport_targetIsModule(t *testing.T) {
-	defer testChdir(t, testFixturePath("import-missing-resource-config"))()
+	t.Chdir(testFixturePath("import-missing-resource-config"))
 
 	statePath := testTempFile(t)
 
