@@ -67,6 +67,7 @@ type variable struct {
 	Default     json.RawMessage `json:"default,omitempty"`
 	Description string          `json:"description,omitempty"`
 	Sensitive   bool            `json:"sensitive,omitempty"`
+	Deprecated  string          `json:"deprecated,omitempty"`
 }
 
 // Resource is the representation of a resource in the config
@@ -111,6 +112,7 @@ type resource struct {
 
 type output struct {
 	Sensitive   bool       `json:"sensitive,omitempty"`
+	Deprecated  string     `json:"deprecated,omitempty"`
 	Expression  expression `json:"expression,omitempty"`
 	DependsOn   []string   `json:"depends_on,omitempty"`
 	Description string     `json:"description,omitempty"`
@@ -326,6 +328,7 @@ func marshalModule(c *configs.Config, schemas *tofu.Schemas, addr string) (modul
 	for _, v := range c.Module.Outputs {
 		o := output{
 			Sensitive:  v.Sensitive,
+			Deprecated: v.Deprecated,
 			Expression: marshalExpression(v.Expr),
 		}
 		if v.Description != "" {
@@ -367,6 +370,7 @@ func marshalModule(c *configs.Config, schemas *tofu.Schemas, addr string) (modul
 				Default:     defaultValJSON,
 				Description: v.Description,
 				Sensitive:   v.Sensitive,
+				Deprecated:  v.Deprecated,
 			}
 		}
 		module.Variables = vars
