@@ -34,6 +34,7 @@ import (
 	"github.com/opentofu/opentofu/internal/states/statemgr"
 	"github.com/opentofu/opentofu/internal/tfdiags"
 	"github.com/opentofu/opentofu/internal/tofu"
+	"github.com/opentofu/opentofu/internal/tracing"
 
 	backendInit "github.com/opentofu/opentofu/internal/backend/init"
 	backendLocal "github.com/opentofu/opentofu/internal/backend/local"
@@ -91,9 +92,12 @@ type BackendWithRemoteTerraformVersion interface {
 // A side-effect of this method is the population of m.backendState, recording
 // the final resolved backend configuration after dealing with overrides from
 // the "tofu init" command line, etc.
-func (m *Meta) Backend(opts *BackendOpts, enc encryption.StateEncryption) (backend.Enhanced, tfdiags.Diagnostics) {
+func (m *Meta) Backend(ctx context.Context, opts *BackendOpts, enc encryption.StateEncryption) (backend.Enhanced, tfdiags.Diagnostics) {
 	var diags tfdiags.Diagnostics
 
+	ctx, span := tracing.Tracer().Start(ctx, "Initialize Backend")
+	defer span.End()
+
 	// If no opts are set, then initialize
 	if opts == nil {
 		opts = &BackendOpts{}
@@ -104,7 +108,7 @@ func (m *Meta) Backend(opts *BackendOpts, enc encryption.StateEncryption) (backe
 	var b backend.Backend
 	if !opts.ForceLocal {
 		var backendDiags tfdiags.Diagnostics
-		b, backendDiags = m.backendFromConfig(opts, enc)
+		b, backendDiags = m.backendFromConfig(ctx, opts, enc)
 		diags = diags.Append(backendDiags)
 
 		if diags.HasErrors() {
@@ -115,7 +119,7 @@ func (m *Meta) Backend(opts *BackendOpts, enc encryption.StateEncryption) (backe
 	}
 
 	// Set up the CLI opts we pass into backends that support it.
-	cliOpts, err := m.backendCLIOpts()
+	cliOpts, err := m.backendCLIOpts(ctx)
 	if err != nil {
 		if errs := providerPluginErrors(nil); errors.As(err, &errs) {
 			// This is a special type returned by m.providerFactories, which
@@ -216,8 +220,8 @@ func (m *Meta) Backend(opts *BackendOpts, enc encryption.StateEncryption) (backe
 // selectWorkspace gets a list of existing workspaces and then checks
 // if the currently selected workspace is valid. If not, it will ask
 // the user to select a workspace from the list.
-func (m *Meta) selectWorkspace(b backend.Backend) error {
-	workspaces, err := b.Workspaces()
+func (m *Meta) selectWorkspace(ctx context.Context, b backend.Backend) error {
+	workspaces, err := b.Workspaces(ctx)
 	if err == backend.ErrWorkspacesNotSupported {
 		return nil
 	}
@@ -244,12 +248,12 @@ func (m *Meta) selectWorkspace(b backend.Backend) error {
 			log.Printf("[TRACE] Meta.selectWorkspace: selecting the new TFC workspace requested by the user (%s)", name)
 			return m.SetWorkspace(name)
 		} else {
-			return fmt.Errorf(strings.TrimSpace(errBackendNoExistingWorkspaces))
+			return fmt.Errorf("%s", strings.TrimSpace(errBackendNoExistingWorkspaces))
 		}
 	}
 
 	// Get the currently selected workspace.
-	workspace, err := m.Workspace()
+	workspace, err := m.Workspace(ctx)
 	if err != nil {
 		return err
 	}
@@ -304,7 +308,7 @@ func (m *Meta) selectWorkspace(b backend.Backend) error {
 // The current workspace name is also stored as part of the plan, and so this
 // method will check that it matches the currently-selected workspace name
 // and produce error diagnostics if not.
-func (m *Meta) BackendForLocalPlan(settings plans.Backend, enc encryption.StateEncryption) (backend.Enhanced, tfdiags.Diagnostics) {
+func (m *Meta) BackendForLocalPlan(ctx context.Context, settings plans.Backend, enc encryption.StateEncryption) (backend.Enhanced, tfdiags.Diagnostics) {
 	var diags tfdiags.Diagnostics
 
 	f := backendInit.Backend(settings.Type)
@@ -328,7 +332,7 @@ func (m *Meta) BackendForLocalPlan(settings plans.Backend, enc encryption.StateE
 		return nil, diags
 	}
 
-	configureDiags := b.Configure(newVal)
+	configureDiags := b.Configure(ctx, newVal)
 	diags = diags.Append(configureDiags)
 	if configureDiags.HasErrors() {
 		return nil, diags
@@ -336,7 +340,7 @@ func (m *Meta) BackendForLocalPlan(settings plans.Backend, enc encryption.StateE
 
 	// If the backend supports CLI initialization, do it.
 	if cli, ok := b.(backend.CLI); ok {
-		cliOpts, err := m.backendCLIOpts()
+		cliOpts, err := m.backendCLIOpts(ctx)
 		if err != nil {
 			diags = diags.Append(err)
 			return nil, diags
@@ -365,7 +369,7 @@ func (m *Meta) BackendForLocalPlan(settings plans.Backend, enc encryption.StateE
 	// Otherwise, we'll wrap our state-only remote backend in the local backend
 	// to cause any operations to be run locally.
 	log.Printf("[TRACE] Meta.BackendForLocalPlan: backend %T does not support operations, so wrapping it in a local backend", b)
-	cliOpts, err := m.backendCLIOpts()
+	cliOpts, err := m.backendCLIOpts(ctx)
 	if err != nil {
 		diags = diags.Append(err)
 		return nil, diags
@@ -382,8 +386,8 @@ func (m *Meta) BackendForLocalPlan(settings plans.Backend, enc encryption.StateE
 
 // backendCLIOpts returns a backend.CLIOpts object that should be passed to
 // a backend that supports local CLI operations.
-func (m *Meta) backendCLIOpts() (*backend.CLIOpts, error) {
-	contextOpts, err := m.contextOpts()
+func (m *Meta) backendCLIOpts(ctx context.Context) (*backend.CLIOpts, error) {
+	contextOpts, err := m.contextOpts(ctx)
 	if contextOpts == nil && err != nil {
 		return nil, err
 	}
@@ -405,9 +409,9 @@ func (m *Meta) backendCLIOpts() (*backend.CLIOpts, error) {
 // This prepares the operation. After calling this, the caller is expected
 // to modify fields of the operation such as Sequence to specify what will
 // be called.
-func (m *Meta) Operation(b backend.Backend, vt arguments.ViewType, enc encryption.Encryption) *backend.Operation {
+func (m *Meta) Operation(ctx context.Context, b backend.Backend, vt arguments.ViewType, enc encryption.Encryption) *backend.Operation {
 	schema := b.ConfigSchema()
-	workspace, err := m.Workspace()
+	workspace, err := m.Workspace(ctx)
 	if err != nil {
 		// An invalid workspace error would have been raised when creating the
 		// backend, and the caller should have already exited. Seeing the error
@@ -453,12 +457,12 @@ func (m *Meta) Operation(b backend.Backend, vt arguments.ViewType, enc encryptio
 }
 
 // backendConfig returns the local configuration for the backend
-func (m *Meta) backendConfig(opts *BackendOpts) (*configs.Backend, int, tfdiags.Diagnostics) {
+func (m *Meta) backendConfig(ctx context.Context, opts *BackendOpts) (*configs.Backend, int, tfdiags.Diagnostics) {
 	var diags tfdiags.Diagnostics
 
 	if opts.Config == nil {
 		// check if the config was missing, or just not required
-		conf, moreDiags := m.loadBackendConfig(".")
+		conf, moreDiags := m.loadBackendConfig(ctx, ".")
 		diags = diags.Append(moreDiags)
 		if moreDiags.HasErrors() {
 			return nil, 0, diags
@@ -532,9 +536,9 @@ func (m *Meta) backendConfig(opts *BackendOpts) (*configs.Backend, int, tfdiags.
 //
 // This function may query the user for input unless input is disabled, in
 // which case this function will error.
-func (m *Meta) backendFromConfig(opts *BackendOpts, enc encryption.StateEncryption) (backend.Backend, tfdiags.Diagnostics) {
+func (m *Meta) backendFromConfig(ctx context.Context, opts *BackendOpts, enc encryption.StateEncryption) (backend.Backend, tfdiags.Diagnostics) {
 	// Get the local backend configuration.
-	c, cHash, diags := m.backendConfig(opts)
+	c, cHash, diags := m.backendConfig(ctx, opts)
 	if diags.HasErrors() {
 		return nil, diags
 	}
@@ -631,7 +635,7 @@ func (m *Meta) backendFromConfig(opts *BackendOpts, enc encryption.StateEncrypti
 			return nil, diags
 		}
 
-		return m.backend_c_r_S(c, cHash, sMgr, true, opts, enc)
+		return m.backend_c_r_S(ctx, c, cHash, sMgr, true, opts, enc)
 
 	// Configuring a backend for the first time or -reconfigure flag was used
 	case c != nil && s.Backend.Empty():
@@ -654,7 +658,7 @@ func (m *Meta) backendFromConfig(opts *BackendOpts, enc encryption.StateEncrypti
 			}
 			return nil, diags
 		}
-		return m.backend_C_r_s(c, cHash, sMgr, opts, enc)
+		return m.backend_C_r_s(ctx, c, cHash, sMgr, opts, enc)
 	// Potentially changing a backend configuration
 	case c != nil && !s.Backend.Empty():
 		// We are not going to migrate if...
@@ -667,7 +671,7 @@ func (m *Meta) backendFromConfig(opts *BackendOpts, enc encryption.StateEncrypti
 			savedBackend, diags := m.savedBackend(sMgr, enc)
 			// Verify that selected workspace exist. Otherwise prompt user to create one
 			if opts.Init && savedBackend != nil {
-				if err := m.selectWorkspace(savedBackend); err != nil {
+				if err := m.selectWorkspace(ctx, savedBackend); err != nil {
 					diags = diags.Append(err)
 					return nil, diags
 				}
@@ -696,7 +700,7 @@ func (m *Meta) backendFromConfig(opts *BackendOpts, enc encryption.StateEncrypti
 			}
 			// Verify that selected workspace exist. Otherwise prompt user to create one
 			if opts.Init && savedBackend != nil {
-				if err := m.selectWorkspace(savedBackend); err != nil {
+				if err := m.selectWorkspace(ctx, savedBackend); err != nil {
 					diags = diags.Append(err)
 					return nil, diags
 				}
@@ -721,7 +725,7 @@ func (m *Meta) backendFromConfig(opts *BackendOpts, enc encryption.StateEncrypti
 		}
 
 		log.Printf("[WARN] backend config has changed since last init")
-		return m.backend_C_r_S_changed(c, cHash, sMgr, true, opts, enc)
+		return m.backend_C_r_S_changed(ctx, c, cHash, sMgr, true, opts, enc)
 
 	default:
 		diags = diags.Append(fmt.Errorf(
@@ -838,7 +842,7 @@ func (m *Meta) backendFromState(ctx context.Context, enc encryption.StateEncrypt
 		return nil, diags
 	}
 
-	configDiags := b.Configure(newVal)
+	configDiags := b.Configure(ctx, newVal)
 	diags = diags.Append(configDiags)
 	if configDiags.HasErrors() {
 		return nil, diags
@@ -877,6 +881,7 @@ func (m *Meta) backendFromState(ctx context.Context, enc encryption.StateEncrypt
 
 // Unconfiguring a backend (moving from backend => local).
 func (m *Meta) backend_c_r_S(
+	ctx context.Context,
 	c *configs.Backend, cHash int, sMgr *clistate.LocalState, output bool, opts *BackendOpts, enc encryption.StateEncryption) (backend.Backend, tfdiags.Diagnostics) {
 
 	var diags tfdiags.Diagnostics
@@ -906,7 +911,7 @@ func (m *Meta) backend_c_r_S(
 	}
 
 	// Grab a purely local backend to get the local state if it exists
-	localB, moreDiags := m.Backend(&BackendOpts{ForceLocal: true, Init: true}, enc)
+	localB, moreDiags := m.Backend(ctx, &BackendOpts{ForceLocal: true, Init: true}, enc)
 	diags = diags.Append(moreDiags)
 	if moreDiags.HasErrors() {
 		return nil, diags
@@ -920,7 +925,7 @@ func (m *Meta) backend_c_r_S(
 	}
 
 	// Perform the migration
-	err := m.backendMigrateState(&backendMigrateOpts{
+	err := m.backendMigrateState(ctx, &backendMigrateOpts{
 		SourceType:      s.Backend.Type,
 		DestinationType: "local",
 		Source:          b,
@@ -954,7 +959,7 @@ func (m *Meta) backend_c_r_S(
 }
 
 // Configuring a backend for the first time.
-func (m *Meta) backend_C_r_s(c *configs.Backend, cHash int, sMgr *clistate.LocalState, opts *BackendOpts, enc encryption.StateEncryption) (backend.Backend, tfdiags.Diagnostics) {
+func (m *Meta) backend_C_r_s(ctx context.Context, c *configs.Backend, cHash int, sMgr *clistate.LocalState, opts *BackendOpts, enc encryption.StateEncryption) (backend.Backend, tfdiags.Diagnostics) {
 	var diags tfdiags.Diagnostics
 
 	vt := arguments.ViewJSON
@@ -965,13 +970,13 @@ func (m *Meta) backend_C_r_s(c *configs.Backend, cHash int, sMgr *clistate.Local
 	}
 
 	// Grab a purely local backend to get the local state if it exists
-	localB, localBDiags := m.Backend(&BackendOpts{ForceLocal: true, Init: true}, enc)
+	localB, localBDiags := m.Backend(ctx, &BackendOpts{ForceLocal: true, Init: true}, enc)
 	if localBDiags.HasErrors() {
 		diags = diags.Append(localBDiags)
 		return nil, diags
 	}
 
-	workspaces, err := localB.Workspaces()
+	workspaces, err := localB.Workspaces(ctx)
 	if err != nil {
 		diags = diags.Append(fmt.Errorf(errBackendLocalRead, err))
 		return nil, diags
@@ -979,7 +984,7 @@ func (m *Meta) backend_C_r_s(c *configs.Backend, cHash int, sMgr *clistate.Local
 
 	var localStates []statemgr.Full
 	for _, workspace := range workspaces {
-		localState, err := localB.StateMgr(workspace)
+		localState, err := localB.StateMgr(ctx, workspace)
 		if err != nil {
 			diags = diags.Append(fmt.Errorf(errBackendLocalRead, err))
 			return nil, diags
@@ -1013,7 +1018,7 @@ func (m *Meta) backend_C_r_s(c *configs.Backend, cHash int, sMgr *clistate.Local
 
 	if len(localStates) > 0 {
 		// Perform the migration
-		err = m.backendMigrateState(&backendMigrateOpts{
+		err = m.backendMigrateState(ctx, &backendMigrateOpts{
 			SourceType:      "local",
 			DestinationType: c.Type,
 			Source:          localB,
@@ -1084,7 +1089,7 @@ func (m *Meta) backend_C_r_s(c *configs.Backend, cHash int, sMgr *clistate.Local
 
 	// Verify that selected workspace exists in the backend.
 	if opts.Init && b != nil {
-		err := m.selectWorkspace(b)
+		err := m.selectWorkspace(ctx, b)
 		if err != nil {
 			diags = diags.Append(err)
 
@@ -1123,7 +1128,7 @@ func (m *Meta) backend_C_r_s(c *configs.Backend, cHash int, sMgr *clistate.Local
 }
 
 // Changing a previously saved backend.
-func (m *Meta) backend_C_r_S_changed(c *configs.Backend, cHash int, sMgr *clistate.LocalState, output bool, opts *BackendOpts, enc encryption.StateEncryption) (backend.Backend, tfdiags.Diagnostics) {
+func (m *Meta) backend_C_r_S_changed(ctx context.Context, c *configs.Backend, cHash int, sMgr *clistate.LocalState, output bool, opts *BackendOpts, enc encryption.StateEncryption) (backend.Backend, tfdiags.Diagnostics) {
 	var diags tfdiags.Diagnostics
 
 	vt := arguments.ViewJSON
@@ -1187,7 +1192,7 @@ func (m *Meta) backend_C_r_S_changed(c *configs.Backend, cHash int, sMgr *clista
 		}
 
 		// Perform the migration
-		err := m.backendMigrateState(&backendMigrateOpts{
+		err := m.backendMigrateState(ctx, &backendMigrateOpts{
 			SourceType:      s.Backend.Type,
 			DestinationType: c.Type,
 			Source:          oldB,
@@ -1229,7 +1234,7 @@ func (m *Meta) backend_C_r_S_changed(c *configs.Backend, cHash int, sMgr *clista
 
 	// Verify that selected workspace exist. Otherwise prompt user to create one
 	if opts.Init && b != nil {
-		if err := m.selectWorkspace(b); err != nil {
+		if err := m.selectWorkspace(ctx, b); err != nil {
 			diags = diags.Append(err)
 			return b, diags
 		}
@@ -1295,7 +1300,7 @@ func (m *Meta) savedBackend(sMgr *clistate.LocalState, enc encryption.StateEncry
 		return nil, diags
 	}
 
-	configDiags := b.Configure(newVal)
+	configDiags := b.Configure(context.TODO(), newVal)
 	diags = diags.Append(configDiags)
 	if configDiags.HasErrors() {
 		return nil, diags
@@ -1440,7 +1445,7 @@ func (m *Meta) backendInitFromConfig(c *configs.Backend, enc encryption.StateEnc
 		return nil, cty.NilVal, diags
 	}
 
-	configureDiags := b.Configure(newVal)
+	configureDiags := b.Configure(context.TODO(), newVal)
 	diags = diags.Append(configureDiags.InConfigBody(c.Config, ""))
 
 	// If the result of loading the backend is an enhanced backend,
