@@ -35,6 +35,13 @@ func NewUiHook(view *View) *UiHook {
 	}
 }
 
+func NewUIOptionalHook(view *View) tofu.Hook {
+	if view.concise {
+		return &tofu.NilHook{}
+	}
+	return NewUiHook(view)
+}
+
 type UiHook struct {
 	tofu.NilHook
 
@@ -276,11 +283,9 @@ func (h *UiHook) PreRefresh(addr addrs.AbsResourceInstance, gen states.Generatio
 	if depKey, ok := gen.(states.DeposedKey); ok {
 		addrStr = fmt.Sprintf("%s (deposed object %s)", addrStr, depKey)
 	}
-	if !h.view.concise {
-		h.println(fmt.Sprintf(
-			h.view.colorize.Color("[reset][bold]%s: Refreshing state...%s"),
-			addrStr, stateIdSuffix))
-	}
+	h.println(fmt.Sprintf(
+		h.view.colorize.Color("[reset][bold]%s: Refreshing state...%s"),
+		addrStr, stateIdSuffix))
 	return tofu.HookActionContinue, nil
 }
 
