@@ -282,6 +282,7 @@ func TestJSONView_ChangeSummary(t *testing.T) {
 				"import":    float64(0),
 				"change":    float64(2),
 				"remove":    float64(3),
+				"forget":    float64(0),
 				"operation": "apply",
 			},
 		},
@@ -312,6 +313,38 @@ func TestJSONView_ChangeSummaryWithImport(t *testing.T) {
 				"change":    float64(2),
 				"remove":    float64(3),
 				"import":    float64(1),
+				"forget":    float64(0),
+				"operation": "apply",
+			},
+		},
+	}
+	testJSONViewOutputEquals(t, done(t).Stdout(), want)
+}
+
+func TestJSONView_ChangeSummaryWithForget(t *testing.T) {
+	streams, done := terminal.StreamsForTesting(t)
+	jv := NewJSONView(NewView(streams))
+
+	jv.ChangeSummary(&viewsjson.ChangeSummary{
+		Add:       1,
+		Change:    2,
+		Remove:    3,
+		Forget:    1,
+		Operation: viewsjson.OperationApplied,
+	})
+
+	want := []map[string]interface{}{
+		{
+			"@level":   "info",
+			"@message": "Apply complete! Resources: 1 added, 2 changed, 3 destroyed, 1 forgotten.",
+			"@module":  "tofu.ui",
+			"type":     "change_summary",
+			"changes": map[string]interface{}{
+				"add":       float64(1),
+				"change":    float64(2),
+				"remove":    float64(3),
+				"import":    float64(0),
+				"forget":    float64(1),
 				"operation": "apply",
 			},
 		},
