@@ -489,8 +489,10 @@ func validateProviderConfigs(parentCall *ModuleCall, cfg *Config, noProviderConf
 				})
 			}
 
-			instanceExpr := instanced[providerName(passed.InParent.Name, passed.InParent.Alias)]
-			diags = diags.Extend(passed.InParent.InstanceValidation("module", instanceExpr != nil))
+			pn := providerName(passed.InParent.Name, passed.InParent.Alias)
+			_, hasConfig := configured[pn]
+			instanceExpr := instanced[pn]
+			diags = diags.Extend(passed.InParent.InstanceValidation("module", instanceExpr != nil, hasConfig))
 			// We could theoretically check here if there are resources (ignoring data blocks) within this submodule graph.
 			// The foot-gun only exists in that scenario, but the complexity of differentiating at the moment is not worth it
 			if passed.InParent.KeyExpression != nil {
@@ -506,8 +508,10 @@ func validateProviderConfigs(parentCall *ModuleCall, cfg *Config, noProviderConf
 				continue
 			}
 
-			instanceExpr := instanced[providerName(r.ProviderConfigRef.Name, r.ProviderConfigRef.Alias)]
-			diags = diags.Extend(r.ProviderConfigRef.InstanceValidation("resource", instanceExpr != nil))
+			pn := providerName(r.ProviderConfigRef.Name, r.ProviderConfigRef.Alias)
+			_, hasConfig := configured[pn]
+			instanceExpr := instanced[pn]
+			diags = diags.Extend(r.ProviderConfigRef.InstanceValidation("resource", instanceExpr != nil, hasConfig))
 			if r.ProviderConfigRef.KeyExpression != nil && r.Mode == addrs.ManagedResourceMode {
 				addr := fmt.Sprintf("%s.%s", r.Type, r.Name)
 				if !cfg.Path.IsRoot() {
@@ -541,7 +545,7 @@ func validateProviderConfigs(parentCall *ModuleCall, cfg *Config, noProviderConf
 			// This name was not declared somewhere within in the
 			// configuration. We ignore empty configs, because they will
 			// already produce a warning.
-			if !(confOK || localOK) {
+			if !confOK && !localOK {
 				defAddr := addrs.NewDefaultProvider(name)
 				diags = append(diags, &hcl.Diagnostic{
 					Severity: hcl.DiagWarning,
@@ -559,7 +563,7 @@ func validateProviderConfigs(parentCall *ModuleCall, cfg *Config, noProviderConf
 			// Now we may have named this provider within the module, but
 			// there won't be a configuration available at runtime if the
 			// parent module did not pass one in.
-			if !cfg.Path.IsRoot() && !(confOK || passedOK) {
+			if !cfg.Path.IsRoot() && !confOK && !passedOK {
 				defAddr := addrs.NewDefaultProvider(name)
 				diags = append(diags, &hcl.Diagnostic{
 					Severity: hcl.DiagWarning,
@@ -664,7 +668,7 @@ func validateProviderConfigs(parentCall *ModuleCall, cfg *Config, noProviderConf
 
 		_, emptyConfig := emptyConfigs[name]
 
-		if !(localName || configAlias || emptyConfig) {
+		if !localName && !configAlias && !emptyConfig {
 
 			// we still allow default configs, so switch to a warning if the incoming provider is a default
 			if addrs.IsDefaultProvider(providerAddr.Provider) {
