@@ -157,8 +157,8 @@ type TestRun struct {
 	// Module defines an address of another module that should be loaded and
 	// executed as part of this run block instead of the module under test.
 	//
-	// In the initial version of the testing framework we will only support
-	// loading alternate modules from local directories or the registry.
+	// We support loading from all the module sources, like local directories, registry,
+	// generic git repos, github, bitbucket, s3 and gcs repositories.
 	Module *TestRunModuleCall
 
 	// ConfigUnderTest describes the configuration this run block should execute
@@ -716,19 +716,6 @@ func decodeTestRunModuleBlock(block *hcl.Block) (*TestRunModuleCall, hcl.Diagnos
 					}
 				}
 			}
-
-			switch module.Source.(type) {
-			case addrs.ModuleSourceRemote:
-				// We only support local or registry modules when loading
-				// modules directly from alternate sources during a test
-				// execution.
-				diags = append(diags, &hcl.Diagnostic{
-					Severity: hcl.DiagError,
-					Summary:  "Invalid module source address",
-					Detail:   "Only local or registry module sources are currently supported from within test run blocks.",
-					Subject:  module.SourceDeclRange.Ptr(),
-				})
-			}
 		}
 	} else {
 		// Must have a source attribute.
@@ -1136,7 +1123,6 @@ var testRunModuleBlockSchema = &hcl.BodySchema{
 	},
 }
 
-//nolint:gochecknoglobals // To follow existing code style.
 var overrideResourceBlockSchema = &hcl.BodySchema{
 	Attributes: []hcl.AttributeSchema{
 		{
@@ -1150,7 +1136,6 @@ var overrideResourceBlockSchema = &hcl.BodySchema{
 	},
 }
 
-//nolint:gochecknoglobals // To follow existing code style.
 var overrideModuleBlockSchema = &hcl.BodySchema{
 	Attributes: []hcl.AttributeSchema{
 		{
@@ -1164,7 +1149,6 @@ var overrideModuleBlockSchema = &hcl.BodySchema{
 	},
 }
 
-//nolint:gochecknoglobals // To follow existing code style.
 var mockProviderBlockSchema = &hcl.BodySchema{
 	Attributes: []hcl.AttributeSchema{
 		{
@@ -1190,7 +1174,6 @@ var mockProviderBlockSchema = &hcl.BodySchema{
 	},
 }
 
-//nolint:gochecknoglobals // To follow existing code style.
 var mockResourceBlockSchema = &hcl.BodySchema{
 	Attributes: []hcl.AttributeSchema{
 		{
