@@ -11,8 +11,8 @@ import "github.com/zclconf/go-cty/cty"
 // It contains two keys because some key providers may prefer include random data (e.g. salt)
 // in the generated keys and this salt will be different for decryption and encryption.
 type Output struct {
-	EncryptionKey []byte `hcl:"encryption_key" cty:"encryption_key" json:"encryption_key" yaml:"encryption_key"`
-	DecryptionKey []byte `hcl:"decryption_key" cty:"decryption_key" json:"decryption_key" yaml:"decryption_key"`
+	EncryptionKey []byte `hcl:"encryption_key" cty:"encryption_key" json:"encryption_key,omitempty" yaml:"encryption_key"`
+	DecryptionKey []byte `hcl:"decryption_key,optional" cty:"decryption_key" json:"decryption_key,omitempty" yaml:"decryption_key"`
 }
 
 // Cty turns the Output struct into a CTY value.
