@@ -15,9 +15,11 @@ import (
 
 	"github.com/google/go-cmp/cmp"
 	version "github.com/hashicorp/go-version"
+
 	"github.com/opentofu/opentofu/internal/configs"
 	"github.com/opentofu/opentofu/internal/configs/configload"
 	"github.com/opentofu/opentofu/internal/copy"
+	"github.com/opentofu/opentofu/internal/getmodules"
 	"github.com/opentofu/opentofu/internal/registry"
 	"github.com/opentofu/opentofu/internal/tfdiags"
 )
@@ -28,8 +30,7 @@ func TestDirFromModule_registry(t *testing.T) {
 	}
 
 	fixtureDir := filepath.Clean("testdata/empty")
-	tmpDir, done := tempChdir(t, fixtureDir)
-	defer done()
+	tmpDir := tempChdir(t, fixtureDir)
 
 	// the module installer runs filepath.EvalSymlinks() on the destination
 	// directory before copying files, and the resultant directory is what is
@@ -43,10 +44,9 @@ func TestDirFromModule_registry(t *testing.T) {
 
 	hooks := &testInstallHooks{}
 
-	reg := registry.NewClient(nil, nil)
-	loader, cleanup := configload.NewLoaderForTests(t)
-	defer cleanup()
-	diags := DirFromModule(context.Background(), loader, dir, modsDir, "hashicorp/module-installer-acctest/aws//examples/main", reg, hooks)
+	reg := registry.NewClient(t.Context(), nil, nil)
+	loader := configload.NewLoaderForTests(t)
+	diags := DirFromModule(context.Background(), loader, dir, modsDir, "hashicorp/module-installer-acctest/aws//examples/main", reg, nil, hooks)
 	assertNoDiagnostics(t, diags)
 
 	v := version.Must(version.NewVersion("0.0.2"))
@@ -110,7 +110,7 @@ func TestDirFromModule_registry(t *testing.T) {
 
 	// Make sure the configuration is loadable now.
 	// (This ensures that correct information is recorded in the manifest.)
-	config, loadDiags := loader.LoadConfig(".", configs.RootModuleCallForTesting())
+	config, loadDiags := loader.LoadConfig(t.Context(), ".", configs.RootModuleCallForTesting())
 	if assertNoDiagnostics(t, tfdiags.Diagnostics{}.Append(loadDiags)) {
 		return
 	}
@@ -152,8 +152,7 @@ func TestDirFromModule_submodules(t *testing.T) {
 		t.Error(err)
 	}
 
-	tmpDir, done := tempChdir(t, fixtureDir)
-	defer done()
+	tmpDir := tempChdir(t, fixtureDir)
 
 	hooks := &testInstallHooks{}
 	dir, err := filepath.EvalSymlinks(tmpDir)
@@ -162,9 +161,21 @@ func TestDirFromModule_submodules(t *testing.T) {
 	}
 	modInstallDir := filepath.Join(dir, ".terraform/modules")
 
-	loader, cleanup := configload.NewLoaderForTests(t)
-	defer cleanup()
-	diags := DirFromModule(context.Background(), loader, dir, modInstallDir, fromModuleDir, nil, hooks)
+	loader := configload.NewLoaderForTests(t)
+	diags := DirFromModule(
+		context.Background(),
+		loader,
+		dir,
+		modInstallDir,
+		fromModuleDir,
+		nil,
+		// This test relies on the module installer's legacy support for
+		// treating an absolute filesystem path as if it were a "remote"
+		// source address, and so we need a real package fetcher but the
+		// way we use it here does not cause it to make network requests.
+		getmodules.NewPackageFetcher(t.Context(), nil),
+		hooks,
+	)
 	assertNoDiagnostics(t, diags)
 	wantCalls := []testInstallHookCall{
 		{
@@ -192,7 +203,7 @@ func TestDirFromModule_submodules(t *testing.T) {
 
 	// Make sure the configuration is loadable now.
 	// (This ensures that correct information is recorded in the manifest.)
-	config, loadDiags := loader.LoadConfig(".", configs.RootModuleCallForTesting())
+	config, loadDiags := loader.LoadConfig(t.Context(), ".", configs.RootModuleCallForTesting())
 	if assertNoDiagnostics(t, tfdiags.Diagnostics{}.Append(loadDiags)) {
 		return
 	}
@@ -226,9 +237,7 @@ func TestDirFromModule_submodulesWithProvider(t *testing.T) {
 		t.Fatal(err)
 	}
 
-	tmpDir, done := tempChdir(t, fixtureDir)
-	defer done()
-
+	tmpDir := tempChdir(t, fixtureDir)
 	hooks := &testInstallHooks{}
 	dir, err := filepath.EvalSymlinks(tmpDir)
 	if err != nil {
@@ -236,9 +245,21 @@ func TestDirFromModule_submodulesWithProvider(t *testing.T) {
 	}
 	modInstallDir := filepath.Join(dir, ".terraform/modules")
 
-	loader, cleanup := configload.NewLoaderForTests(t)
-	defer cleanup()
-	diags := DirFromModule(context.Background(), loader, dir, modInstallDir, fromModuleDir, nil, hooks)
+	loader := configload.NewLoaderForTests(t)
+	diags := DirFromModule(
+		context.Background(),
+		loader,
+		dir,
+		modInstallDir,
+		fromModuleDir,
+		nil,
+		// This test relies on the module installer's legacy support for
+		// treating an absolute filesystem path as if it were a "remote"
+		// source address, and so we need a real package fetcher but the
+		// way we use it here does not cause it to make network requests.
+		getmodules.NewPackageFetcher(t.Context(), nil),
+		hooks,
+	)
 
 	for _, d := range diags {
 		if d.Severity() != tfdiags.Warning {
@@ -293,9 +314,20 @@ func TestDirFromModule_rel_submodules(t *testing.T) {
 
 	modInstallDir := ".terraform/modules"
 	sourceDir := "../local-modules"
-	loader, cleanup := configload.NewLoaderForTests(t)
-	defer cleanup()
-	diags := DirFromModule(context.Background(), loader, ".", modInstallDir, sourceDir, nil, hooks)
+	loader := configload.NewLoaderForTests(t)
+	diags := DirFromModule(
+		context.Background(),
+		loader, ".",
+		modInstallDir,
+		sourceDir,
+		nil,
+		// This test relies on the module installer's legacy support for
+		// treating an absolute filesystem path as if it were a "remote"
+		// source address, and so we need a real package fetcher but the
+		// way we use it here does not cause it to make network requests.
+		getmodules.NewPackageFetcher(t.Context(), nil),
+		hooks,
+	)
 	assertNoDiagnostics(t, diags)
 	wantCalls := []testInstallHookCall{
 		{
@@ -323,7 +355,7 @@ func TestDirFromModule_rel_submodules(t *testing.T) {
 
 	// Make sure the configuration is loadable now.
 	// (This ensures that correct information is recorded in the manifest.)
-	config, loadDiags := loader.LoadConfig(".", configs.RootModuleCallForTesting())
+	config, loadDiags := loader.LoadConfig(t.Context(), ".", configs.RootModuleCallForTesting())
 	if assertNoDiagnostics(t, tfdiags.Diagnostics{}.Append(loadDiags)) {
 		return
 	}
