@@ -20,6 +20,8 @@ import (
 	version "github.com/hashicorp/go-version"
 	"github.com/hashicorp/hcl/v2"
 	"github.com/hashicorp/hcl/v2/hclsyntax"
+	otelAttr "go.opentelemetry.io/otel/attribute"
+	"go.opentelemetry.io/otel/trace"
 
 	"github.com/opentofu/opentofu/internal/addrs"
 	"github.com/opentofu/opentofu/internal/configs"
@@ -30,12 +32,15 @@ import (
 	"github.com/opentofu/opentofu/internal/registry/regsrc"
 	"github.com/opentofu/opentofu/internal/registry/response"
 	"github.com/opentofu/opentofu/internal/tfdiags"
+	"github.com/opentofu/opentofu/internal/tracing"
+	"github.com/opentofu/opentofu/internal/tracing/traceattrs"
 )
 
 type ModuleInstaller struct {
 	modsDir string
 	loader  *configload.Loader
 	reg     *registry.Client
+	fetcher *getmodules.PackageFetcher
 
 	// The keys in moduleVersions are resolved and trimmed registry source
 	// addresses and the values are the registry response.
@@ -51,11 +56,30 @@ type moduleVersion struct {
 	version string
 }
 
-func NewModuleInstaller(modsDir string, loader *configload.Loader, reg *registry.Client) *ModuleInstaller {
+// NewModuleInstaller constructs a new [ModuleInstaller] object whose methods
+// will make use of the given dependencies.
+//
+// "loader" is the configuration loader to use to traverse the module tree
+// of the configuration whose modules are being installed.
+//
+// "registryClient" is the client for the OpenTofu module registry protocol,
+// used to fetch package metadata when installing remote modules indirectly
+// through a registry-style module source address. This may be nil only if
+// "remotePackageFetcher" is also nil, since registry source addresses are
+// only resolvable when remote module packages are available.
+//
+// "remotePackageFetcher" is the client used for fetching actual module packages
+// from concrete physical source locations, which can be either specified
+// directly in the configuration or returned dynamically as part of the metadata
+// fetched from an OpenTofu module registry. This argument can be nil, in which
+// case no remote package sources are supported; this facility is included
+// primarily for unit testing where only local modules are needed.
+func NewModuleInstaller(modsDir string, loader *configload.Loader, registryClient *registry.Client, remotePackageFetcher *getmodules.PackageFetcher) *ModuleInstaller {
 	return &ModuleInstaller{
 		modsDir:                 modsDir,
 		loader:                  loader,
-		reg:                     reg,
+		reg:                     registryClient,
+		fetcher:                 remotePackageFetcher,
 		registryPackageVersions: make(map[addrs.ModuleRegistryPackage]*response.ModuleVersions),
 		registryPackageSources:  make(map[moduleVersion]addrs.ModuleSourceRemote),
 	}
@@ -124,7 +148,7 @@ func (i *ModuleInstaller) InstallModules(ctx context.Context, rootDir, testsDir
 		return nil, diags
 	}
 
-	fetcher := getmodules.NewPackageFetcher()
+	fetcher := i.fetcher
 
 	if hooks == nil {
 		// Use our no-op implementation as a placeholder
@@ -139,15 +163,15 @@ func (i *ModuleInstaller) InstallModules(ctx context.Context, rootDir, testsDir
 	}
 	walker := i.moduleInstallWalker(ctx, manifest, upgrade, hooks, fetcher)
 
-	cfg, instDiags := i.installDescendentModules(rootMod, manifest, walker, installErrsOnly)
+	cfg, instDiags := i.installDescendentModules(ctx, rootMod, manifest, walker, installErrsOnly)
 	diags = append(diags, instDiags...)
 
 	return cfg, diags
 }
 
-func (i *ModuleInstaller) moduleInstallWalker(ctx context.Context, manifest modsdir.Manifest, upgrade bool, hooks ModuleInstallHooks, fetcher *getmodules.PackageFetcher) configs.ModuleWalker {
+func (i *ModuleInstaller) moduleInstallWalker(_ context.Context, manifest modsdir.Manifest, upgrade bool, hooks ModuleInstallHooks, fetcher *getmodules.PackageFetcher) configs.ModuleWalker {
 	return configs.ModuleWalkerFunc(
-		func(req *configs.ModuleRequest) (*configs.Module, *version.Version, hcl.Diagnostics) {
+		func(ctx context.Context, req *configs.ModuleRequest) (*configs.Module, *version.Version, hcl.Diagnostics) {
 			var diags hcl.Diagnostics
 
 			if req.SourceAddr == nil {
@@ -176,6 +200,14 @@ func (i *ModuleInstaller) moduleInstallWalker(ctx context.Context, manifest mods
 			key := manifest.ModuleKey(req.Path)
 			instPath := i.packageInstallPath(req.Path)
 
+			ctx, span := tracing.Tracer().Start(ctx,
+				fmt.Sprintf("Install Module %q", req.Name),
+				trace.WithAttributes(
+					otelAttr.String(traceattrs.ModuleCallName, req.Name),
+					otelAttr.String(traceattrs.ModuleSource, req.SourceAddr.String()),
+				))
+			defer span.End()
+
 			log.Printf("[DEBUG] Module installer: begin %s", key)
 
 			// First we'll check if we need to upgrade/replace an existing
@@ -186,12 +218,15 @@ func (i *ModuleInstaller) moduleInstallWalker(ctx context.Context, manifest mods
 				switch {
 				case !recorded:
 					log.Printf("[TRACE] ModuleInstaller: %s is not yet installed", key)
+					span.AddEvent("Module not yet installed")
 					replace = true
 				case record.SourceAddr != req.SourceAddr.String():
 					log.Printf("[TRACE] ModuleInstaller: %s source address has changed from %q to %q", key, record.SourceAddr, req.SourceAddr)
+					span.AddEvent("Module source address changed")
 					replace = true
 				case record.Version != nil && !req.VersionConstraint.Required.Check(record.Version):
 					log.Printf("[TRACE] ModuleInstaller: %s version %s no longer compatible with constraints %s", key, record.Version, req.VersionConstraint.Required)
+					span.AddEvent("Module version constraint changed")
 					replace = true
 				}
 			}
@@ -268,13 +303,15 @@ func (i *ModuleInstaller) moduleInstallWalker(ctx context.Context, manifest mods
 
 			case addrs.ModuleSourceLocal:
 				log.Printf("[TRACE] ModuleInstaller: %s has local path %q", key, addr.String())
-				mod, mDiags := i.installLocalModule(req, key, manifest, hooks)
+				span.SetAttributes(otelAttr.String("opentofu.module.source_type", "local"))
+				mod, mDiags := i.installLocalModule(ctx, req, key, manifest, hooks)
 				mDiags = maybeImproveLocalInstallError(req, mDiags)
 				diags = append(diags, mDiags...)
 				return mod, nil, diags
 
 			case addrs.ModuleSourceRegistry:
 				log.Printf("[TRACE] ModuleInstaller: %s is a registry module at %s", key, addr.String())
+				span.SetAttributes(otelAttr.String("opentofu.module.source_type", "registry"))
 				mod, v, mDiags := i.installRegistryModule(ctx, req, key, instPath, addr, manifest, hooks, fetcher)
 				diags = append(diags, mDiags...)
 				return mod, v, diags
@@ -294,7 +331,7 @@ func (i *ModuleInstaller) moduleInstallWalker(ctx context.Context, manifest mods
 	)
 }
 
-func (i *ModuleInstaller) installDescendentModules(rootMod *configs.Module, manifest modsdir.Manifest, installWalker configs.ModuleWalker, installErrsOnly bool) (*configs.Config, tfdiags.Diagnostics) {
+func (i *ModuleInstaller) installDescendentModules(ctx context.Context, rootMod *configs.Module, manifest modsdir.Manifest, installWalker configs.ModuleWalker, installErrsOnly bool) (*configs.Config, tfdiags.Diagnostics) {
 	var diags tfdiags.Diagnostics
 
 	// When attempting to initialize the current directory with a module
@@ -307,14 +344,14 @@ func (i *ModuleInstaller) installDescendentModules(rootMod *configs.Module, mani
 	var instDiags hcl.Diagnostics
 	walker := installWalker
 	if installErrsOnly {
-		walker = configs.ModuleWalkerFunc(func(req *configs.ModuleRequest) (*configs.Module, *version.Version, hcl.Diagnostics) {
-			mod, version, diags := installWalker.LoadModule(req)
+		walker = configs.ModuleWalkerFunc(func(ctx context.Context, req *configs.ModuleRequest) (*configs.Module, *version.Version, hcl.Diagnostics) {
+			mod, version, diags := installWalker.LoadModule(ctx, req)
 			instDiags = instDiags.Extend(diags)
 			return mod, version, diags
 		})
 	}
 
-	cfg, cDiags := configs.BuildConfig(rootMod, walker)
+	cfg, cDiags := configs.BuildConfig(ctx, rootMod, walker)
 	diags = diags.Append(cDiags)
 	if installErrsOnly {
 		// We can't continue if there was an error during installation, but
@@ -346,9 +383,15 @@ func (i *ModuleInstaller) installDescendentModules(rootMod *configs.Module, mani
 	return cfg, diags
 }
 
-func (i *ModuleInstaller) installLocalModule(req *configs.ModuleRequest, key string, manifest modsdir.Manifest, hooks ModuleInstallHooks) (*configs.Module, hcl.Diagnostics) {
+func (i *ModuleInstaller) installLocalModule(ctx context.Context, req *configs.ModuleRequest, key string, manifest modsdir.Manifest, hooks ModuleInstallHooks) (*configs.Module, hcl.Diagnostics) {
 	var diags hcl.Diagnostics
 
+	_, span := tracing.Tracer().Start(ctx, "Install Local Module",
+		trace.WithAttributes(otelAttr.String(traceattrs.ModuleCallName, req.Name)),
+		trace.WithAttributes(otelAttr.String(traceattrs.ModuleSource, req.SourceAddr.String())),
+	)
+	defer span.End()
+
 	parentKey := manifest.ModuleKey(req.Parent.Path)
 	parentRecord, recorded := manifest[parentKey]
 	if !recorded {
@@ -401,6 +444,10 @@ func (i *ModuleInstaller) installLocalModule(req *configs.ModuleRequest, key str
 		diags = diags.Extend(mDiags)
 	}
 
+	if diags.HasErrors() {
+		tracing.SetSpanError(span, diags)
+	}
+
 	// Note the local location in our manifest.
 	manifest[key] = modsdir.Record{
 		Key:        key,
@@ -421,6 +468,27 @@ var versionRegexp = regexp.MustCompile(version.VersionRegexpRaw)
 func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *configs.ModuleRequest, key string, instPath string, addr addrs.ModuleSourceRegistry, manifest modsdir.Manifest, hooks ModuleInstallHooks, fetcher *getmodules.PackageFetcher) (*configs.Module, *version.Version, hcl.Diagnostics) {
 	var diags hcl.Diagnostics
 
+	ctx, span := tracing.Tracer().Start(ctx, "Install Registry Module",
+		trace.WithAttributes(otelAttr.String(traceattrs.ModuleCallName, req.Name)),
+		trace.WithAttributes(otelAttr.String(traceattrs.ModuleSource, req.SourceAddr.String())),
+		trace.WithAttributes(otelAttr.String(traceattrs.ModuleVersion, req.VersionConstraint.Required.String())),
+	)
+	defer span.End()
+
+	if i.reg == nil || fetcher == nil {
+		// Only local package sources are available when we have no registry
+		// client or no fetcher, since both would be needed for successful install.
+		// (This special situation is primarily for use in tests.)
+		diags = diags.Append(&hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Registry-style module sources not supported",
+			Detail:   "Only local module sources are supported in this context.",
+			Subject:  req.CallRange.Ptr(),
+		})
+		tracing.SetSpanError(span, diags)
+		return nil, nil, diags
+	}
+
 	hostname := addr.Package.Host
 	reg := i.reg
 	var resp *response.ModuleVersions
@@ -469,6 +537,7 @@ func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *config
 					Subject:  req.CallRange.Ptr(),
 				})
 			}
+			tracing.SetSpanError(span, diags)
 			return nil, nil, diags
 		}
 		i.registryPackageVersions[packageAddr] = resp
@@ -618,6 +687,7 @@ func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *config
 			Detail:   fmt.Sprintf("Module %q (%s:%d) has no versions available on %s.", addr, req.CallRange.Filename, req.CallRange.Start.Line, hostname),
 			Subject:  req.CallRange.Ptr(),
 		})
+		tracing.SetSpanError(span, diags)
 		return nil, nil, diags
 	}
 
@@ -628,6 +698,7 @@ func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *config
 			Detail:   fmt.Sprintf("There is no available version of module %q (%s:%d) which matches the given version constraint. The newest available version is %s.", addr, req.CallRange.Filename, req.CallRange.Start.Line, latestVersion),
 			Subject:  req.CallRange.Ptr(),
 		})
+		tracing.SetSpanError(span, diags)
 		return nil, nil, diags
 	}
 
@@ -649,6 +720,7 @@ func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *config
 				Summary:  "Error accessing remote module registry",
 				Detail:   fmt.Sprintf("Failed to retrieve a download URL for %s %s from %s: %s", addr, latestMatch, hostname, err),
 			})
+			tracing.SetSpanError(span, diags)
 			return nil, nil, diags
 		}
 		realAddr, err := addrs.ParseModuleSource(realAddrRaw)
@@ -658,8 +730,12 @@ func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *config
 				Summary:  "Invalid package location from module registry",
 				Detail:   fmt.Sprintf("Module registry %s returned invalid source location %q for %s %s: %s.", hostname, realAddrRaw, addr, latestMatch, err),
 			})
+			tracing.SetSpanError(span, diags)
 			return nil, nil, diags
 		}
+
+		span.SetAttributes(otelAttr.String(traceattrs.ModuleSource, realAddr.String()))
+
 		switch realAddr := realAddr.(type) {
 		// Only a remote source address is allowed here: a registry isn't
 		// allowed to return a local path (because it doesn't know what
@@ -673,6 +749,7 @@ func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *config
 				Summary:  "Invalid package location from module registry",
 				Detail:   fmt.Sprintf("Module registry %s returned invalid source location %q for %s %s: must be a direct remote package address.", hostname, realAddrRaw, addr, latestMatch),
 			})
+			tracing.SetSpanError(span, diags)
 			return nil, nil, diags
 		}
 	}
@@ -754,6 +831,18 @@ func (i *ModuleInstaller) installRegistryModule(ctx context.Context, req *config
 func (i *ModuleInstaller) installGoGetterModule(ctx context.Context, req *configs.ModuleRequest, key string, instPath string, manifest modsdir.Manifest, hooks ModuleInstallHooks, fetcher *getmodules.PackageFetcher) (*configs.Module, hcl.Diagnostics) {
 	var diags hcl.Diagnostics
 
+	if fetcher == nil {
+		// Only local package sources are available when we have no fetcher.
+		// (This special situation is primarily for use in tests.)
+		diags = diags.Append(&hcl.Diagnostic{
+			Severity: hcl.DiagError,
+			Summary:  "Remote module sources not supported",
+			Detail:   "Only local module sources are supported in this context.",
+			Subject:  req.CallRange.Ptr(),
+		})
+		return nil, diags
+	}
+
 	// Report up to the caller that we're about to start downloading.
 	addr := req.SourceAddr.(addrs.ModuleSourceRemote)
 	packageAddr := addr.Package
