@@ -9,15 +9,20 @@ import (
 	"context"
 	"fmt"
 	"log"
+	"slices"
 	"sort"
 	"strings"
 
 	"github.com/apparentlymart/go-versions/versions"
+	otelAttr "go.opentelemetry.io/otel/attribute"
+	"go.opentelemetry.io/otel/trace"
 
 	"github.com/opentofu/opentofu/internal/addrs"
 	copydir "github.com/opentofu/opentofu/internal/copy"
 	"github.com/opentofu/opentofu/internal/depsfile"
 	"github.com/opentofu/opentofu/internal/getproviders"
+	"github.com/opentofu/opentofu/internal/tracing"
+	"github.com/opentofu/opentofu/internal/tracing/traceattrs"
 )
 
 // Installer is the main type in this package, representing a provider installer
@@ -182,9 +187,23 @@ func (i *Installer) SetUnmanagedProviderTypes(types map[addrs.Provider]struct{})
 // in the final returned error value so callers should show either one or the
 // other, and not both.
 func (i *Installer) EnsureProviderVersions(ctx context.Context, locks *depsfile.Locks, reqs getproviders.Requirements, mode InstallMode) (*depsfile.Locks, error) {
-	errs := map[addrs.Provider]error{}
+	ctx, span := tracing.Tracer().Start(ctx, "Install Providers") // TODO: Discuss span name
+	defer span.End()
+
 	evts := installerEventsForContext(ctx)
 
+	// Whenever possible we prefer to collect separate errors for each
+	// problematic provider and then report them all together at the end,
+	// because that can allow an operator to notice a systematic problem
+	// across multiple providers, such as a particular registry failing
+	// in the same way regardless of which provider is requested.
+	//
+	// The other functions we call below will gradually add errors here
+	// as appropriate. Those functions only return an err directly
+	// themselves in situations that are not related to any particular
+	// provider and so prevent us from continuing further at all.
+	errs := map[addrs.Provider]error{}
+
 	// We'll work with a copy of the given locks, so we can modify it and
 	// return the updated locks without affecting the caller's object.
 	// We'll add, replace, or remove locks in here during our work so that the
@@ -201,8 +220,67 @@ func (i *Installer) EnsureProviderVersions(ctx context.Context, locks *depsfile.
 	// just ask the source to confirm the continued existence of what
 	// was locked, or otherwise we'll find the newest version matching the
 	// configured version constraint.
+	mightNeed, locked := i.ensureProviderVersionsMightNeed(ctx, locks, reqs, mode, errs)
+
+	// Step 2: Query the provider source for each of the providers we selected
+	// in the first step and select the latest available version that is
+	// in the set of acceptable versions.
+	//
+	// This produces a set of packages to install to our cache in the next step.
+	need, err := i.ensureProviderVersionsNeed(ctx, locks, reqs, mightNeed, locked, errs)
+	if err != nil {
+		return nil, err
+	}
+
+	// Step 3: For each provider version we've decided we need to install,
+	// install its package into our target cache (possibly via the global cache).
+	targetPlatform := i.targetDir.targetPlatform // we inherit this to behave correctly in unit tests
+	span.SetAttributes(otelAttr.String(traceattrs.TargetPlatform, targetPlatform.String()))
+	span.SetName("Install Providers - " + targetPlatform.String())
+	authResults, err := i.ensureProviderVersionsInstall(ctx, locks, reqs, mode, need, targetPlatform, errs)
+	if err != nil {
+		return nil, err
+	}
+
+	// Emit final event for fetching if any were successfully fetched
+	if cb := evts.ProvidersAuthenticated; cb != nil && len(authResults) > 0 {
+		cb(authResults)
+	}
+
+	// Finally, if the lock structure contains locks for any providers that
+	// are no longer needed by this configuration, we'll remove them. This
+	// is important because we will not have installed those providers
+	// above and so a lock file still containing them would make the working
+	// directory invalid: not every provider in the lock file is available
+	// for use.
+	for providerAddr := range locks.AllProviders() {
+		if _, ok := reqs[providerAddr]; !ok {
+			locks.RemoveProvider(providerAddr)
+		}
+	}
+
+	if len(errs) > 0 {
+		return locks, InstallerError{
+			ProviderErrors: errs,
+		}
+	}
+	return locks, nil
+}
+
+func (i *Installer) ensureProviderVersionsMightNeed(
+	ctx context.Context,
+	locks *depsfile.Locks,
+	reqs getproviders.Requirements,
+	mode InstallMode,
+	errs map[addrs.Provider]error,
+) (
+	map[addrs.Provider]getproviders.VersionSet,
+	map[addrs.Provider]bool,
+) {
+	evts := installerEventsForContext(ctx)
 	mightNeed := map[addrs.Provider]getproviders.VersionSet{}
 	locked := map[addrs.Provider]bool{}
+
 	for provider, versionConstraints := range reqs {
 		if provider.IsBuiltIn() {
 			// Built in providers do not require installation but we'll still
@@ -277,13 +355,21 @@ func (i *Installer) EnsureProviderVersions(ctx context.Context, locks *depsfile.
 		mightNeed[provider] = acceptableVersions
 	}
 
-	// Step 2: Query the provider source for each of the providers we selected
-	// in the first step and select the latest available version that is
-	// in the set of acceptable versions.
-	//
-	// This produces a set of packages to install to our cache in the next step.
+	return mightNeed, locked
+}
+
+func (i *Installer) ensureProviderVersionsNeed(
+	ctx context.Context,
+	locks *depsfile.Locks,
+	reqs getproviders.Requirements,
+	mightNeed map[addrs.Provider]getproviders.VersionSet,
+	locked map[addrs.Provider]bool,
+	errs map[addrs.Provider]error,
+) (map[addrs.Provider]getproviders.Version, error) {
+	evts := installerEventsForContext(ctx)
 	need := map[addrs.Provider]getproviders.Version{}
 NeedProvider:
+
 	for provider, acceptableVersions := range mightNeed {
 		if err := ctx.Err(); err != nil {
 			// If our context has been cancelled or reached a timeout then
@@ -346,402 +432,323 @@ NeedProvider:
 		}
 	}
 
-	// Step 3: For each provider version we've decided we need to install,
-	// install its package into our target cache (possibly via the global cache).
+	return need, nil
+}
+
+func (i *Installer) ensureProviderVersionsInstall(
+	ctx context.Context,
+	locks *depsfile.Locks,
+	reqs getproviders.Requirements,
+	mode InstallMode,
+	need map[addrs.Provider]getproviders.Version,
+	targetPlatform getproviders.Platform,
+	errs map[addrs.Provider]error,
+) (map[addrs.Provider]*getproviders.PackageAuthenticationResult, error) {
 	authResults := map[addrs.Provider]*getproviders.PackageAuthenticationResult{} // record auth results for all successfully fetched providers
-	targetPlatform := i.targetDir.targetPlatform                                  // we inherit this to behave correctly in unit tests
+
 	for provider, version := range need {
-		if err := ctx.Err(); err != nil {
+		traceCtx, span := tracing.Tracer().Start(ctx,
+			fmt.Sprintf("Install Provider %q", provider.String()),
+			trace.WithAttributes(
+				otelAttr.String(traceattrs.ProviderAddress, provider.String()),
+				otelAttr.String(traceattrs.ProviderVersion, version.String()),
+				otelAttr.String(traceattrs.TargetPlatform, targetPlatform.String()),
+			),
+		)
+
+		if err := traceCtx.Err(); err != nil {
 			// If our context has been cancelled or reached a timeout then
 			// we'll abort early, because subsequent operations against
 			// that context will fail immediately anyway.
+			tracing.SetSpanError(span, err)
+			span.End()
 			return nil, err
 		}
 
-		lock := locks.Provider(provider)
-		var preferredHashes []getproviders.Hash
-		if lock != nil && lock.Version() == version { // hash changes are expected if the version is also changing
-			preferredHashes = lock.PreferredHashes()
+		authResult, err := i.ensureProviderVersionInstalled(traceCtx, locks, reqs, mode, provider, version, targetPlatform)
+		if authResult != nil {
+			authResults[provider] = authResult
 		}
-
-		// If our target directory already has the provider version that fulfills the lock file, carry on
-		if installed := i.targetDir.ProviderVersion(provider, version); installed != nil {
-			if len(preferredHashes) > 0 {
-				if matches, _ := installed.MatchesAnyHash(preferredHashes); matches {
-					if cb := evts.ProviderAlreadyInstalled; cb != nil {
-						cb(provider, version)
-					}
-					continue
-				}
-			}
-		}
-
-		if i.globalCacheDir != nil {
-			// Step 3a: If our global cache already has this version available then
-			// we'll just link it in.
-			if cached := i.globalCacheDir.ProviderVersion(provider, version); cached != nil {
-				// An existing cache entry is only an acceptable choice
-				// if there is already a lock file entry for this provider
-				// and the cache entry matches its checksums.
-				//
-				// If there was no lock file entry at all then we need to
-				// install the package for real so that we can lock as complete
-				// as possible a set of checksums for all of this provider's
-				// packages.
-				//
-				// If there was a lock file entry but the cache doesn't match
-				// it then we assume that the lock file checksums were only
-				// partially populated (e.g. from a local mirror where we can
-				// only see one package to checksum it) and so we'll fetch
-				// from upstream to see if the origin can give us a package
-				// that _does_ match. This might still not work out, but if
-				// it does then it allows us to avoid returning a checksum
-				// mismatch error.
-				acceptablePackage := false
-				if len(preferredHashes) != 0 {
-					var err error
-					acceptablePackage, err = cached.MatchesAnyHash(preferredHashes)
-					if err != nil {
-						// If we can't calculate the checksum for the cached
-						// package then we'll just treat it as a checksum failure.
-						acceptablePackage = false
-					}
-				}
-
-				if !acceptablePackage && i.globalCacheDirMayBreakDependencyLockFile {
-					// The "may break dependency lock file" setting effectively
-					// means that we'll accept any matching package that's
-					// already in the cache, regardless of whether it matches
-					// what's in the dependency lock file.
-					//
-					// That means two less-ideal situations might occur:
-					// - If this provider is not currently tracked in the lock
-					//   file at all then after installation the lock file will
-					//   only accept the package that was already present in
-					//   the cache as a valid checksum. That means the generated
-					//   lock file won't be portable to other operating systems
-					//   or CPU architectures.
-					// - If the provider _is_ currently tracked in the lock file
-					//   but the checksums there don't match what was in the
-					//   cache then the LinkFromOtherCache call below will
-					//   fail with a checksum error, and the user will need to
-					//   either manually remove the entry from the lock file
-					//   or remove the mismatching item from the cache,
-					//   depending on which of these they prefer to use as the
-					//   source of truth for the expected contents of the
-					//   package.
-					//
-					// If the lock file already includes this provider and the
-					// cache entry matches one of the locked checksums then
-					// there's no problem, but in that case we wouldn't enter
-					// this branch because acceptablePackage would already be
-					// true from the check above.
-					log.Printf(
-						"[WARN] plugin_cache_may_break_dependency_lock_file: Using global cache dir package for %s v%s even though it doesn't match this configuration's dependency lock file",
-						provider.String(), version.String(),
-					)
-					acceptablePackage = true
-				}
-
-				// TODO: Should we emit an event through the events object
-				// for "there was an entry in the cache but we ignored it
-				// because the checksum didn't match"? We can't use
-				// LinkFromCacheFailure in that case because this isn't a
-				// failure. For now we'll just be quiet about it.
-
-				if acceptablePackage {
-					if cb := evts.LinkFromCacheBegin; cb != nil {
-						cb(provider, version, i.globalCacheDir.baseDir)
-					}
-					if _, err := cached.ExecutableFile(); err != nil {
-						err := fmt.Errorf("provider binary not found: %w", err)
-						errs[provider] = err
-						if cb := evts.LinkFromCacheFailure; cb != nil {
-							cb(provider, version, err)
-						}
-						continue
-					}
-
-					err := i.targetDir.LinkFromOtherCache(cached, preferredHashes)
-					if err != nil {
-						errs[provider] = err
-						if cb := evts.LinkFromCacheFailure; cb != nil {
-							cb(provider, version, err)
-						}
-						continue
-					}
-					// We'll fetch what we just linked to make sure it actually
-					// did show up there.
-					new := i.targetDir.ProviderVersion(provider, version)
-					if new == nil {
-						err := fmt.Errorf("after linking %s from provider cache at %s it is still not detected in the target directory; this is a bug in OpenTofu", provider, i.globalCacheDir.baseDir)
-						errs[provider] = err
-						if cb := evts.LinkFromCacheFailure; cb != nil {
-							cb(provider, version, err)
-						}
-						continue
-					}
-
-					// The LinkFromOtherCache call above should've verified that
-					// the package matches one of the hashes previously recorded,
-					// if any. We'll now augment those hashes with one freshly
-					// calculated from the package we just linked, which allows
-					// the lock file to gradually transition to recording newer hash
-					// schemes when they become available.
-					var priorHashes []getproviders.Hash
-					if lock != nil && lock.Version() == version {
-						// If the version we're installing is identical to the
-						// one we previously locked then we'll keep all of the
-						// hashes we saved previously and add to it. Otherwise
-						// we'll be starting fresh, because each version has its
-						// own set of packages and thus its own hashes.
-						priorHashes = append(priorHashes, preferredHashes...)
-
-						// NOTE: The behavior here is unfortunate when a particular
-						// provider version was already cached on the first time
-						// the current configuration requested it, because that
-						// means we don't currently get the opportunity to fetch
-						// and verify the checksums for the new package from
-						// upstream. That's currently unavoidable because upstream
-						// checksums are in the "ziphash" format and so we can't
-						// verify them against our cache directory's unpacked
-						// packages: we'd need to go fetch the package from the
-						// origin and compare against it, which would defeat the
-						// purpose of the global cache.
-						//
-						// If we fetch from upstream on the first encounter with
-						// a particular provider then we'll end up in the other
-						// codepath below where we're able to also include the
-						// checksums from the origin registry.
-					}
-					newHash, err := cached.Hash()
-					if err != nil {
-						err := fmt.Errorf("after linking %s from provider cache at %s, failed to compute a checksum for it: %w", provider, i.globalCacheDir.baseDir, err)
-						errs[provider] = err
-						if cb := evts.LinkFromCacheFailure; cb != nil {
-							cb(provider, version, err)
-						}
-						continue
-					}
-					// The hashes slice gets deduplicated in the lock file
-					// implementation, so we don't worry about potentially
-					// creating a duplicate here.
-					var newHashes []getproviders.Hash
-					newHashes = append(newHashes, priorHashes...)
-					newHashes = append(newHashes, newHash)
-					locks.SetProvider(provider, version, reqs[provider], newHashes)
-					if cb := evts.ProvidersLockUpdated; cb != nil {
-						// We want to ensure that newHash and priorHashes are
-						// sorted. newHash is a single value, so it's definitely
-						// sorted. priorHashes are pulled from the lock file, so
-						// are also already sorted.
-						cb(provider, version, []getproviders.Hash{newHash}, nil, priorHashes)
-					}
-
-					if cb := evts.LinkFromCacheSuccess; cb != nil {
-						cb(provider, version, new.PackageDir)
-					}
-					continue // Don't need to do full install, then.
-				}
-			}
-		}
-
-		// Step 3b: Get the package metadata for the selected version from our
-		// provider source.
-		//
-		// This is the step where we might detect and report that the provider
-		// isn't available for the current platform.
-		if cb := evts.FetchPackageMeta; cb != nil {
-			cb(provider, version)
-		}
-		meta, err := i.source.PackageMeta(ctx, provider, version, targetPlatform)
 		if err != nil {
 			errs[provider] = err
-			if cb := evts.FetchPackageFailure; cb != nil {
-				cb(provider, version, err)
+		}
+		span.End()
+	}
+	return authResults, nil
+}
+
+func (i *Installer) ensureProviderVersionInstalled(
+	ctx context.Context,
+	locks *depsfile.Locks,
+	reqs getproviders.Requirements,
+	mode InstallMode,
+	provider addrs.Provider,
+	version getproviders.Version,
+	targetPlatform getproviders.Platform,
+) (*getproviders.PackageAuthenticationResult, error) {
+	evts := installerEventsForContext(ctx)
+	lock := locks.Provider(provider)
+
+	var preferredHashes []getproviders.Hash
+	if lock != nil && lock.Version() == version { // hash changes are expected if the version is also changing
+		preferredHashes = lock.PreferredHashes()
+	}
+
+	// If our target directory already has the provider version that fulfills the lock file, carry on
+	if installed := i.targetDir.ProviderVersion(provider, version); installed != nil {
+		if len(preferredHashes) > 0 {
+			if matches, _ := installed.MatchesAnyHash(preferredHashes); matches {
+				if cb := evts.ProviderAlreadyInstalled; cb != nil {
+					cb(provider, version, false)
+				}
+
+				// Even though the package is installed, the requirements in the lockfile may still need to be updated
+				locks.SetProvider(provider, version, reqs[provider], lock.AllHashes())
+
+				return nil, nil
 			}
-			continue
+		}
+	}
+
+	var installTo, linkTo *Dir
+	if i.globalCacheDir != nil {
+		installTo = i.globalCacheDir
+		linkTo = i.targetDir
+	} else {
+		installTo = i.targetDir
+		linkTo = nil // no linking needed
+	}
+
+	result, err := i.ensureProviderVersionInDirectory(ctx, locks, reqs, mode, provider, version, targetPlatform, installTo)
+
+	if err != nil {
+		return result, err
+	}
+
+	if linkTo != nil {
+		if cb := evts.LinkFromCacheBegin; cb != nil {
+			cb(provider, version, installTo.BasePath())
 		}
 
-		// Step 3c: Retrieve the package indicated by the metadata we received,
-		// either directly into our target directory or via the global cache
-		// directory.
-		if cb := evts.FetchPackageBegin; cb != nil {
-			cb(provider, version, meta.Location)
-		}
-		var installTo, linkTo *Dir
-		if i.globalCacheDir != nil {
-			installTo = i.globalCacheDir
-			linkTo = i.targetDir
-		} else {
-			installTo = i.targetDir
-			linkTo = nil // no linking needed
-		}
-
-		allowedHashes := preferredHashes
-		if mode.forceInstallChecksums() {
-			allowedHashes = []getproviders.Hash{}
-		}
-
-		authResult, err := installTo.InstallPackage(ctx, meta, allowedHashes)
-		if err != nil {
-			// TODO: Consider retrying for certain kinds of error that seem
-			// likely to be transient. For now, we just treat all errors equally.
-			errs[provider] = err
-			if cb := evts.FetchPackageFailure; cb != nil {
-				cb(provider, version, err)
-			}
-			continue
-		}
+		// We don't do a hash check here because we already did that
+		// as part of the ensureProviderVersionInDirectory call above.
 		new := installTo.ProviderVersion(provider, version)
-		if new == nil {
-			err := fmt.Errorf("after installing %s it is still not detected in %s; this is a bug in OpenTofu", provider, installTo.BasePath())
-			errs[provider] = err
-			if cb := evts.FetchPackageFailure; cb != nil {
+		err := linkTo.LinkFromOtherCache(ctx, new, nil)
+		if err != nil {
+			if cb := evts.LinkFromCacheFailure; cb != nil {
 				cb(provider, version, err)
 			}
-			continue
+			return nil, err
+		}
+
+		// We should now also find the package in the linkTo dir, which
+		// gives us the final value of "new" where the path points in to
+		// the true target directory, rather than possibly the global
+		// cache directory.
+		new = linkTo.ProviderVersion(provider, version)
+		if new == nil {
+			err := fmt.Errorf("after installing %s it is still not detected in %s; this is a bug in OpenTofu", provider, linkTo.BasePath())
+			if cb := evts.LinkFromCacheFailure; cb != nil {
+				cb(provider, version, err)
+			}
+			return nil, err
 		}
 		if _, err := new.ExecutableFile(); err != nil {
 			err := fmt.Errorf("provider binary not found: %w", err)
-			errs[provider] = err
-			if cb := evts.FetchPackageFailure; cb != nil {
+			if cb := evts.LinkFromCacheFailure; cb != nil {
 				cb(provider, version, err)
 			}
-			continue
+			return nil, err
 		}
-		if linkTo != nil {
-			// We skip emitting the "LinkFromCache..." events here because
-			// it's simpler for the caller to treat them as mutually exclusive.
-			// We can just subsume the linking step under the "FetchPackage..."
-			// series here (and that's why we use FetchPackageFailure below).
-			// We also don't do a hash check here because we already did that
-			// as part of the installTo.InstallPackage call above.
-			err := linkTo.LinkFromOtherCache(new, nil)
-			if err != nil {
-				errs[provider] = err
-				if cb := evts.FetchPackageFailure; cb != nil {
-					cb(provider, version, err)
+
+		if cb := evts.LinkFromCacheSuccess; cb != nil {
+			cb(provider, version, new.PackageDir)
+		}
+	}
+
+	return result, err
+}
+
+func (i *Installer) ensureProviderVersionInDirectory(
+	ctx context.Context,
+	locks *depsfile.Locks,
+	reqs getproviders.Requirements,
+	mode InstallMode,
+	provider addrs.Provider,
+	version getproviders.Version,
+	targetPlatform getproviders.Platform,
+	installTo *Dir,
+) (*getproviders.PackageAuthenticationResult, error) {
+	evts := installerEventsForContext(ctx)
+	lock := locks.Provider(provider)
+
+	var preferredHashes []getproviders.Hash
+	if lock != nil && lock.Version() == version { // hash changes are expected if the version is also changing
+		preferredHashes = lock.PreferredHashes()
+	}
+
+	isGlobalCache := installTo == i.globalCacheDir
+
+	// If our target directory already has the provider version that fulfills the lock file, carry on
+	if installed := installTo.ProviderVersion(provider, version); installed != nil {
+		if len(preferredHashes) > 0 {
+			if matches, _ := installed.MatchesAnyHash(preferredHashes); matches {
+				if cb := evts.ProviderAlreadyInstalled; cb != nil {
+					cb(provider, version, isGlobalCache)
 				}
-				continue
+
+				// Even though the package is installed, the requirements in the lockfile may still need to be updated
+				locks.SetProvider(provider, version, reqs[provider], lock.AllHashes())
+
+				return nil, nil
 			}
-
-			// We should now also find the package in the linkTo dir, which
-			// gives us the final value of "new" where the path points in to
-			// the true target directory, rather than possibly the global
-			// cache directory.
-			new = linkTo.ProviderVersion(provider, version)
-			if new == nil {
-				err := fmt.Errorf("after installing %s it is still not detected in %s; this is a bug in OpenTofu", provider, linkTo.BasePath())
-				errs[provider] = err
-				if cb := evts.FetchPackageFailure; cb != nil {
-					cb(provider, version, err)
-				}
-				continue
-			}
-			if _, err := new.ExecutableFile(); err != nil {
-				err := fmt.Errorf("provider binary not found: %w", err)
-				errs[provider] = err
-				if cb := evts.FetchPackageFailure; cb != nil {
-					cb(provider, version, err)
-				}
-				continue
-			}
-		}
-		authResults[provider] = authResult
-
-		// The InstallPackage call above should've verified that
-		// the package matches one of the hashes previously recorded,
-		// if any. We'll now augment those hashes with a new set populated
-		// with the hashes returned by the upstream source and from the
-		// package we've just installed, which allows the lock file to
-		// gradually transition to newer hash schemes when they become
-		// available.
-		//
-		// This is assuming that if a package matches both a hash we saw before
-		// _and_ a new hash then the new hash is a valid substitute for
-		// the previous hash.
-		//
-		// The hashes slice gets deduplicated in the lock file
-		// implementation, so we don't worry about potentially
-		// creating duplicates here.
-		var priorHashes []getproviders.Hash
-		if lock != nil && lock.Version() == version {
-			// If the version we're installing is identical to the
-			// one we previously locked then we'll keep all of the
-			// hashes we saved previously and add to it. Otherwise
-			// we'll be starting fresh, because each version has its
-			// own set of packages and thus its own hashes.
-			priorHashes = append(priorHashes, preferredHashes...)
-		}
-		newHash, err := new.Hash()
-		if err != nil {
-			err := fmt.Errorf("after installing %s, failed to compute a checksum for it: %w", provider, err)
-			errs[provider] = err
-			if cb := evts.FetchPackageFailure; cb != nil {
-				cb(provider, version, err)
-			}
-			continue
-		}
-
-		var signedHashes []getproviders.Hash
-		// For now, we will temporarily trust the hashes returned by the
-		// installation process that are "SigningSkipped" or "Signed".
-		// This is only intended to be temporary, see https://github.com/opentofu/opentofu/issues/266 for more information
-		if authResult.Signed() || authResult.SigningSkipped() {
-			// We'll trust new hashes from upstream only if they were verified
-			// as signed by a suitable key or if the signing validation was skipped.
-			// Otherwise, we'd record only
-			// a new hash we just calculated ourselves from the bytes on disk,
-			// and so the hashes would cover only the current platform.
-			signedHashes = append(signedHashes, meta.AcceptableHashes()...)
-		}
-
-		var newHashes []getproviders.Hash
-		newHashes = append(newHashes, newHash)
-		newHashes = append(newHashes, priorHashes...)
-		newHashes = append(newHashes, signedHashes...)
-
-		locks.SetProvider(provider, version, reqs[provider], newHashes)
-		if cb := evts.ProvidersLockUpdated; cb != nil {
-			// newHash and priorHashes are already sorted.
-			// But we do need to sort signedHashes so we can reason about it
-			// sensibly.
-			sort.Slice(signedHashes, func(i, j int) bool {
-				return string(signedHashes[i]) < string(signedHashes[j])
-			})
-
-			cb(provider, version, []getproviders.Hash{newHash}, signedHashes, priorHashes)
-		}
-
-		if cb := evts.FetchPackageSuccess; cb != nil {
-			cb(provider, version, new.PackageDir, authResult)
 		}
 	}
 
-	// Emit final event for fetching if any were successfully fetched
-	if cb := evts.ProvidersFetched; cb != nil && len(authResults) > 0 {
-		cb(authResults)
+	// Step 3b: Get the package metadata for the selected version from our
+	// provider source.
+	//
+	// This is the step where we might detect and report that the provider
+	// isn't available for the current platform.
+	if cb := evts.FetchPackageMeta; cb != nil {
+		cb(provider, version)
+	}
+	meta, err := i.source.PackageMeta(ctx, provider, version, targetPlatform)
+	if err != nil {
+		if cb := evts.FetchPackageFailure; cb != nil {
+			cb(provider, version, err)
+		}
+		return nil, err
 	}
 
-	// Finally, if the lock structure contains locks for any providers that
-	// are no longer needed by this configuration, we'll remove them. This
-	// is important because we will not have installed those providers
-	// above and so a lock file still containing them would make the working
-	// directory invalid: not every provider in the lock file is available
-	// for use.
-	for providerAddr := range locks.AllProviders() {
-		if _, ok := reqs[providerAddr]; !ok {
-			locks.RemoveProvider(providerAddr)
-		}
+	// Step 3c: Retrieve the package indicated by the metadata we received,
+	// either directly into our target directory or via the global cache
+	// directory.
+	if cb := evts.FetchPackageBegin; cb != nil {
+		cb(provider, version, meta.Location, isGlobalCache)
 	}
 
-	if len(errs) > 0 {
-		return locks, InstallerError{
-			ProviderErrors: errs,
-		}
+	allowedHashes := preferredHashes
+	if mode.forceInstallChecksums() {
+		allowedHashes = []getproviders.Hash{}
 	}
-	return locks, nil
+
+	allowSkippingInstallWithoutHashes := i.globalCacheDirMayBreakDependencyLockFile && isGlobalCache
+	authResult, err := installTo.InstallPackage(ctx, meta, allowedHashes, allowSkippingInstallWithoutHashes)
+	if err != nil {
+		// TODO: Consider retrying for certain kinds of error that seem
+		// likely to be transient. For now, we just treat all errors equally.
+		if cb := evts.FetchPackageFailure; cb != nil {
+			cb(provider, version, err)
+		}
+		return nil, err
+	}
+
+	new := installTo.ProviderVersion(provider, version)
+	if new == nil {
+		err := fmt.Errorf("after installing %s it is still not detected in %s; this is a bug in OpenTofu", provider, installTo.BasePath())
+		if cb := evts.FetchPackageFailure; cb != nil {
+			cb(provider, version, err)
+		}
+		return nil, err
+	}
+	if _, err := new.ExecutableFile(); err != nil {
+		err := fmt.Errorf("provider binary not found: %w", err)
+		if cb := evts.FetchPackageFailure; cb != nil {
+			cb(provider, version, err)
+		}
+		return nil, err
+	}
+
+	// The InstallPackage call above should've verified that
+	// the package matches one of the hashes previously recorded,
+	// if any. We'll now augment those hashes with a new set populated
+	// with the hashes returned by the upstream source and from the
+	// package we've just installed, which allows the lock file to
+	// gradually transition to newer hash schemes when they become
+	// available.
+	//
+	// This is assuming that if a package matches both a hash we saw before
+	// _and_ a new hash then the new hash is a valid substitute for
+	// the previous hash.
+	//
+	// The hashes slice gets deduplicated in the lock file
+	// implementation, so we don't worry about potentially
+	// creating duplicates here.
+	var priorHashes []getproviders.Hash
+	if lock != nil && lock.Version() == version {
+		// If the version we're installing is identical to the
+		// one we previously locked then we'll keep all of the
+		// hashes we saved previously and add to it. Otherwise
+		// we'll be starting fresh, because each version has its
+		// own set of packages and thus its own hashes.
+		priorHashes = append(priorHashes, preferredHashes...)
+	}
+	newHash, err := new.Hash()
+	if err != nil {
+		err := fmt.Errorf("after installing %s, failed to compute a checksum for it: %w", provider, err)
+		if cb := evts.FetchPackageFailure; cb != nil {
+			cb(provider, version, err)
+		}
+		return authResult, err
+	}
+
+	// localHashes is the set of hashes that we were able to verify locally
+	// based on the data we downloaded.
+	localHashes := slices.Collect(authResult.HashesWithDisposition(func(hd *getproviders.HashDisposition) bool {
+		return hd.VerifiedLocally
+	}))
+	localHashes = append(localHashes, newHash) // the hash we calculated above was _also_ verified locally
+
+	// We have different rules for what subset of hashes we track in
+	// the dependency lock file depending on the provider. Refer to
+	// the documentation of the following function for more information.
+	signingRequired := getproviders.ShouldEnforceGPGValidationForProvider(provider)
+	signedHashes := slices.Collect(authResult.HashesWithDisposition(func(hd *getproviders.HashDisposition) bool {
+		if !signingRequired {
+			// When signing isn't required, we pretend that anything
+			// that was reported by the origin registry was "signed",
+			// just for the purposes of updating the lock file and
+			// reporting that lock file update to the UI layer through
+			// the evts object.
+			// Note that the "tofu init" UI relies on us pretending
+			// that these are "signed" to avoid generating its warning
+			// that the dependency lock file might be incomplete.
+			return hd.ReportedByRegistry
+		}
+		return hd.SignedByAnyGPGKeys()
+	}))
+
+	var newHashes []getproviders.Hash
+	newHashes = append(newHashes, newHash)
+	newHashes = append(newHashes, priorHashes...)
+	newHashes = append(newHashes, localHashes...)
+	newHashes = append(newHashes, signedHashes...)
+
+	locks.SetProvider(provider, version, reqs[provider], newHashes)
+	if cb := evts.ProvidersLockUpdated; cb != nil {
+		// priorHashes is already sorted, but we do need to sort
+		// the newly-generated localHashes and signedHashes.
+		sort.Slice(localHashes, func(i, j int) bool {
+			return localHashes[i].String() < localHashes[j].String()
+		})
+		sort.Slice(signedHashes, func(i, j int) bool {
+			return signedHashes[i].String() < signedHashes[j].String()
+		})
+		// these slices might also contain duplicates if the
+		// same hash was found in two different ways, so we'll
+		// adjust for that. This relies on the sorting above
+		// and modifies the underlying arrays in-place.
+		localHashes = slices.Compact(localHashes)
+		signedHashes = slices.Compact(signedHashes)
+		cb(provider, version, localHashes, signedHashes, priorHashes)
+	}
+
+	if cb := evts.FetchPackageSuccess; cb != nil {
+		cb(provider, version, new.PackageDir, authResult)
+	}
+
+	return authResult, nil
 }
 
 // checkUnspecifiedVersion Check the presence of version 0.0.0 and return an error with a tip
