@@ -317,6 +317,8 @@ func prepareStateV4(sV4 *stateV4) (*File, tfdiags.Diagnostics) {
 			}
 			os.Sensitive = fos.Sensitive
 
+			os.Deprecated = fos.Deprecated
+
 			ty, err := ctyjson.UnmarshalType([]byte(fos.ValueTypeRaw))
 			if err != nil {
 				diags = diags.Append(tfdiags.Sourceless(
@@ -407,6 +409,7 @@ func writeStateV4(file *File, w io.Writer, enc encryption.StateEncryption) tfdia
 
 		sV4.RootOutputs[name] = outputStateV4{
 			Sensitive:    os.Sensitive,
+			Deprecated:   os.Deprecated,
 			ValueRaw:     json.RawMessage(src),
 			ValueTypeRaw: json.RawMessage(typeSrc),
 		}
@@ -764,6 +767,7 @@ type outputStateV4 struct {
 	ValueRaw     json.RawMessage `json:"value"`
 	ValueTypeRaw json.RawMessage `json:"type"`
 	Sensitive    bool            `json:"sensitive,omitempty"`
+	Deprecated   string          `json:"deprecated,omitempty"`
 }
 
 // Note: the ProviderConfig field is only set on either the resource or the resource instance object
