@@ -123,6 +123,9 @@ func evalCheckRule(addr addrs.CheckRule, rule *configs.CheckRule, ctx EvalContex
 	resultVal, hclDiags := rule.Condition.Value(hclCtx)
 	diags = diags.Append(hclDiags)
 
+	resultVal, deprDiags := marks.ExtractDeprecatedDiagnosticsWithExpr(resultVal, rule.Condition)
+	diags = diags.Append(deprDiags)
+
 	if diags.HasErrors() {
 		log.Printf("[TRACE] evalCheckRule: %s: %s", addr.Type, diags.Err().Error())
 		return checkResult{Status: checks.StatusError}, diags
@@ -230,6 +233,9 @@ func evalCheckErrorMessage(expr hcl.Expression, hclCtx *hcl.EvalContext) (string
 		return "", diags
 	}
 
+	val, deprDiags := marks.ExtractDeprecatedDiagnosticsWithExpr(val, expr)
+	diags = diags.Append(deprDiags)
+
 	val, err := convert.Convert(val, cty.String)
 	if err != nil {
 		diags = diags.Append(&hcl.Diagnostic{
