@@ -92,6 +92,10 @@ type Hook interface {
 	PreApplyImport(addr addrs.AbsResourceInstance, importing plans.ImportingSrc) (HookAction, error)
 	PostApplyImport(addr addrs.AbsResourceInstance, importing plans.ImportingSrc) (HookAction, error)
 
+	// PreApplyForget and PostApplyForget are called during an apply for each forgotten resource.
+	PreApplyForget(addr addrs.AbsResourceInstance) (HookAction, error)
+	PostApplyForget(addr addrs.AbsResourceInstance) (HookAction, error)
+
 	// Stopping is called if an external signal requests that OpenTofu
 	// gracefully abort an operation in progress.
 	//
@@ -188,6 +192,14 @@ func (h *NilHook) PostApplyImport(addr addrs.AbsResourceInstance, importing plan
 	return HookActionContinue, nil
 }
 
+func (h *NilHook) PreApplyForget(_ addrs.AbsResourceInstance) (HookAction, error) {
+	return HookActionContinue, nil
+}
+
+func (h *NilHook) PostApplyForget(_ addrs.AbsResourceInstance) (HookAction, error) {
+	return HookActionContinue, nil
+}
+
 func (*NilHook) Stopping() {
 	// Does nothing at all by default
 }
