@@ -6,37 +6,27 @@
 package tofu
 
 import (
+	"context"
 	"log"
 
-	"github.com/opentofu/opentofu/internal/dag"
 	"github.com/opentofu/opentofu/internal/logging"
 )
 
 // GraphTransformer is the interface that transformers implement. This
 // interface is only for transforms that need entire graph visibility.
 type GraphTransformer interface {
-	Transform(*Graph) error
-}
-
-// GraphVertexTransformer is an interface that transforms a single
-// Vertex within with graph. This is a specialization of GraphTransformer
-// that makes it easy to do vertex replacement.
-//
-// The GraphTransformer that runs through the GraphVertexTransformers is
-// VertexTransformer.
-type GraphVertexTransformer interface {
-	Transform(dag.Vertex) (dag.Vertex, error)
+	Transform(context.Context, *Graph) error
 }
 
 type graphTransformerMulti struct {
 	Transforms []GraphTransformer
 }
 
-func (t *graphTransformerMulti) Transform(g *Graph) error {
+func (t *graphTransformerMulti) Transform(ctx context.Context, g *Graph) error {
 	var lastStepStr string
 	for _, t := range t.Transforms {
 		log.Printf("[TRACE] (graphTransformerMulti) Executing graph transform %T", t)
-		if err := t.Transform(g); err != nil {
+		if err := t.Transform(ctx, g); err != nil {
 			return err
 		}
 
