@@ -6,6 +6,7 @@
 package tofu
 
 import (
+	"context"
 	"log"
 
 	"github.com/opentofu/opentofu/internal/addrs"
@@ -23,7 +24,7 @@ type checkTransformer struct {
 
 var _ GraphTransformer = (*checkTransformer)(nil)
 
-func (t *checkTransformer) Transform(graph *Graph) error {
+func (t *checkTransformer) Transform(_ context.Context, graph *Graph) error {
 	return t.transform(graph, t.Config, graph.Vertices())
 }
 
