@@ -45,12 +45,12 @@ func TestDestroyEdgeTransformer_basic(t *testing.T) {
 		mustProviderConfig(`provider["registry.opentofu.org/hashicorp/test"]`),
 		addrs.NoKey,
 	)
-	if err := (&AttachStateTransformer{State: state}).Transform(&g); err != nil {
+	if err := (&AttachStateTransformer{State: state}).Transform(t.Context(), &g); err != nil {
 		t.Fatal(err)
 	}
 
 	tf := &DestroyEdgeTransformer{}
-	if err := tf.Transform(&g); err != nil {
+	if err := tf.Transform(t.Context(), &g); err != nil {
 		t.Fatalf("err: %s", err)
 	}
 
@@ -102,12 +102,12 @@ func TestDestroyEdgeTransformer_multi(t *testing.T) {
 		addrs.NoKey,
 	)
 
-	if err := (&AttachStateTransformer{State: state}).Transform(&g); err != nil {
+	if err := (&AttachStateTransformer{State: state}).Transform(t.Context(), &g); err != nil {
 		t.Fatal(err)
 	}
 
 	tf := &DestroyEdgeTransformer{}
-	if err := tf.Transform(&g); err != nil {
+	if err := tf.Transform(t.Context(), &g); err != nil {
 		t.Fatalf("err: %s", err)
 	}
 
@@ -122,7 +122,7 @@ func TestDestroyEdgeTransformer_selfRef(t *testing.T) {
 	g := Graph{Path: addrs.RootModuleInstance}
 	g.Add(testDestroyNode("test_object.A"))
 	tf := &DestroyEdgeTransformer{}
-	if err := tf.Transform(&g); err != nil {
+	if err := tf.Transform(t.Context(), &g); err != nil {
 		t.Fatalf("err: %s", err)
 	}
 
@@ -160,12 +160,12 @@ func TestDestroyEdgeTransformer_module(t *testing.T) {
 		addrs.NoKey,
 	)
 
-	if err := (&AttachStateTransformer{State: state}).Transform(&g); err != nil {
+	if err := (&AttachStateTransformer{State: state}).Transform(t.Context(), &g); err != nil {
 		t.Fatal(err)
 	}
 
 	tf := &DestroyEdgeTransformer{}
-	if err := tf.Transform(&g); err != nil {
+	if err := tf.Transform(t.Context(), &g); err != nil {
 		t.Fatalf("err: %s", err)
 	}
 
@@ -222,12 +222,12 @@ func TestDestroyEdgeTransformer_moduleOnly(t *testing.T) {
 		)
 	}
 
-	if err := (&AttachStateTransformer{State: state}).Transform(&g); err != nil {
+	if err := (&AttachStateTransformer{State: state}).Transform(t.Context(), &g); err != nil {
 		t.Fatal(err)
 	}
 
 	tf := &DestroyEdgeTransformer{}
-	if err := tf.Transform(&g); err != nil {
+	if err := tf.Transform(t.Context(), &g); err != nil {
 		t.Fatalf("err: %s", err)
 	}
 
@@ -292,12 +292,12 @@ func TestDestroyEdgeTransformer_destroyThenUpdate(t *testing.T) {
 		addrs.NoKey,
 	)
 
-	if err := (&AttachStateTransformer{State: state}).Transform(&g); err != nil {
+	if err := (&AttachStateTransformer{State: state}).Transform(t.Context(), &g); err != nil {
 		t.Fatal(err)
 	}
 
 	tf := &DestroyEdgeTransformer{}
-	if err := tf.Transform(&g); err != nil {
+	if err := tf.Transform(t.Context(), &g); err != nil {
 		t.Fatalf("err: %s", err)
 	}
 
@@ -407,14 +407,15 @@ func TestPruneUnusedNodesTransformer_rootModuleOutputValues(t *testing.T) {
 				Concrete: concreteResourceInstance,
 				State:    state,
 				Changes:  changes,
+				Config:   config,
 			},
 			&ReferenceTransformer{},
 			&AttachDependenciesTransformer{},
-			&pruneUnusedNodesTransformer{},
+			&pruneUnusedNodesTransformer{Op: walkDestroy},
 			&CloseRootModuleTransformer{},
 		},
 	}
-	graph, diags := builder.Build(addrs.RootModuleInstance)
+	graph, diags := builder.Build(t.Context(), addrs.RootModuleInstance)
 	assertNoDiagnostics(t, diags)
 
 	// At this point, thanks to pruneUnusedNodesTransformer, we should still
@@ -498,7 +499,7 @@ func TestDestroyEdgeTransformer_noOp(t *testing.T) {
 		addrs.NoKey,
 	)
 
-	if err := (&AttachStateTransformer{State: state}).Transform(&g); err != nil {
+	if err := (&AttachStateTransformer{State: state}).Transform(t.Context(), &g); err != nil {
 		t.Fatal(err)
 	}
 
@@ -514,7 +515,7 @@ func TestDestroyEdgeTransformer_noOp(t *testing.T) {
 			},
 		},
 	}
-	if err := tf.Transform(&g); err != nil {
+	if err := tf.Transform(t.Context(), &g); err != nil {
 		t.Fatalf("err: %s", err)
 	}
 
@@ -559,12 +560,12 @@ func TestDestroyEdgeTransformer_dataDependsOn(t *testing.T) {
 		addrs.NoKey,
 	)
 
-	if err := (&AttachStateTransformer{State: state}).Transform(&g); err != nil {
+	if err := (&AttachStateTransformer{State: state}).Transform(t.Context(), &g); err != nil {
 		t.Fatal(err)
 	}
 
 	tf := &DestroyEdgeTransformer{}
-	if err := tf.Transform(&g); err != nil {
+	if err := tf.Transform(t.Context(), &g); err != nil {
 		t.Fatalf("err: %s", err)
 	}
 
